@@ -165,8 +165,15 @@ void NATClient::Stop()
 void NATClient::Exec()
 {
 	ReadLock lockConfig(ConfigReloadMutex);
-	
-	socket = new CallSignalSocket();
+
+#ifdef HAS_TLS
+	if (Toolkit::AsBool(GkConfig()->GetString(EndpointSection, "UseTLS", "0"))) {
+		socket = new TLSCallSignalSocket();
+	} else
+#endif
+	{
+		socket = new CallSignalSocket();
+	}
 	socket->SetPort(gkport);
 	if (socket->Connect(gkip)) {
 		PTRACE(2, "GKC\t" << socket->GetName() << " connected, waiting for incoming call");
@@ -813,7 +820,7 @@ bool STUNClient::CreateSocketPair(const H225_CallIdentifier & id, UDPProxySocket
 
 
 class GkClient;
-class H46024Socket  : public UDPProxySocket
+class H46024Socket : public UDPProxySocket
 {
 public:
     H46024Socket(GkClient * client, bool rtp, const H225_CallIdentifier & id, CallRec::NatStrategy strategy, WORD sessionID);
@@ -847,7 +854,6 @@ public:
 
 	void SetAlternateAddresses(const H323TransportAddress & address, const PString & cui, unsigned muxID);
 	void GetAlternateAddresses(H323TransportAddress & address, PString & cui, unsigned & muxID);
-	PBoolean IsAlternateAddress(const Address & address, WORD port);
 
 	// Annex A
 	PBoolean ReceivedProbePacket(const RTP_ControlFrame & frame, bool & probe, bool & success);
@@ -856,7 +862,6 @@ public:
 	void ProbeReceived(bool probe, const PIPSocket::Address & addr, WORD & port);
 	void SetProbeState(probe_state newstate);
 	int GetProbeState() const;
-	void StartH46024Adirect(bool starter);
 	void SignalH46024Adirect();
 
 	// Annex B
@@ -875,7 +880,7 @@ private:
 	// Addresses
 	PString m_CUIlocal;									///< Local CUI
 	PString m_CUIremote;							    ///< Remote CUI
-	PIPSocket::Address m_locAddr;  WORD m_locPort;		///< local Address (address used when starting socket)
+	PIPSocket::Address m_locAddr;						///< local Address (address used when starting socket)
 	PIPSocket::Address m_remAddr;  WORD m_remPort;		///< Remote Address (address used when starting socket)
 	PIPSocket::Address m_detAddr;  WORD m_detPort;		///< detected remote Address (as detected from actual packets)
 	PIPSocket::Address m_pendAddr;  WORD m_pendPort;	///< detected pending RTCP Probe Address (as detected from actual packets)
@@ -895,9 +900,10 @@ private:
 };
 
 H46024Socket::H46024Socket(GkClient * client, bool rtp, const H225_CallIdentifier & id, CallRec::NatStrategy strategy, WORD sessionID)
-	:UDPProxySocket((rtp ? "rtp" : "rtcp"),id), m_client(client), 
+	:UDPProxySocket((rtp ? "rtp" : "rtcp"), id), m_client(client), 
 	m_natStrategy(strategy), m_sessionID(sessionID), m_callIdentifier(id), 
-	m_rtp(rtp), m_state(e_notRequired), m_CUIlocal(PString()), m_CUIremote(PString()), m_altMuxID(0), m_probes(0), m_keepseqno(100)
+	m_rtp(rtp), m_state(e_notRequired),	m_remPort(0), m_detPort(0), m_pendPort(0), m_altPort(0),
+	m_altMuxID(0), m_probes(0), SSRC(0), m_keepseqno(100)
 {
 
 }
@@ -1047,7 +1053,8 @@ PBoolean H46024Socket::WriteTo(const void * buf, PINDEX len, const Address & add
 PBoolean H46024Socket::WriteSocket(const void * buf, PINDEX len, const Address & addr, WORD port, unsigned altMux)
 {
 	unsigned mux = m_sendMultiplexID;
-	if (altMux) mux = altMux;
+	if (altMux)
+		mux = altMux;
 
 	if (!PNatMethod_H46019::IsMultiplexed() && !mux)      // No Multiplex Rec'v or Send
 		return UDPProxySocket::WriteTo(buf,len, addr, port);
@@ -1090,13 +1097,8 @@ void H46024Socket::SetAlternateAddresses(const H323TransportAddress & address, c
 	}
 }
 
-PBoolean H46024Socket::IsAlternateAddress(const Address & address,WORD port)
-{
-	return ((address == m_detAddr) && (port == m_detPort));
-}
-
-#define H46024A_MAX_PROBE_COUNT  10
-#define H46024A_PROBE_INTERVAL  150
+#define H46024A_MAX_PROBE_COUNT  15
+#define H46024A_PROBE_INTERVAL  200
 void H46024Socket::StartProbe()
 {
 	PTRACE(4, "H46024A\ts: " << m_sessionID << " Starting direct connection probe.");
@@ -1186,20 +1188,6 @@ void H46024Socket::ProbeReceived(bool probe, const PIPSocket::Address & addr, WO
 	}
 }
 
-void H46024Socket::StartH46024Adirect(bool starter)
-{
-	if (GetProbeState() == e_direct)  // We might already be doing Annex B ?
-		return;
-
-	if (starter) {  // We start the direct channel 
-		m_detAddr = m_altAddr;  m_detPort = m_altPort;
-		PTRACE(4, "H46024A\ts:" << m_sessionID << (m_rtp ? " RTP " : " RTCP ")  
-					<< "Switching to " << m_detAddr << ":" << m_detPort);
-		SetProbeState(e_direct);
-	} else         // We wait for the remote to start channel
-		SetProbeState(e_wait);
-}
-
 void H46024Socket::SignalH46024Adirect()
 {
 #ifdef HAS_H46024A
@@ -1465,7 +1453,7 @@ private:
 	unsigned tag;
 };
 
-bool GkClientHandler::IsExpected(const RasMsg *ras) const
+bool GkClientHandler::IsExpected(const RasMsg * ras) const
 {
 	return (ras->GetTag() == tag) && client->CheckFrom(ras);
 }
@@ -1475,13 +1463,12 @@ void GkClientHandler::Process(RasMsg *ras)
 	CreateJob(this, &GkClientHandler::OnRequest, ras, ras->GetTagName());
 }
 
-void GkClientHandler::OnRequest(RasMsg *ras)
+void GkClientHandler::OnRequest(RasMsg * ras)
 {
 	ReadLock lockConfig(ConfigReloadMutex);
 	if ((client->*handlePDU)(ras))
 		ras->Reply();
 	delete ras;
-	ras = NULL;
 }
 
 namespace {
@@ -1516,7 +1503,6 @@ GkClient::GkClient()
 	m_handlers[2] = new GkClientHandler(this, &GkClient::OnDRQ, H225_RasMessage::e_disengageRequest);
 	m_handlers[3] = new GkClientHandler(this, &GkClient::OnIRQ, H225_RasMessage::e_infoRequest);
 
-#ifdef OpenH323Factory
 	m_password = PString::Empty();
     m_h235Authenticators = new H235Authenticators();
     PFactory<H235Authenticator>::KeyList_T keyList = PFactory<H235Authenticator>::GetKeyList();
@@ -1525,15 +1511,11 @@ GkClient::GkClient()
        H235Authenticator * Auth = PFactory<H235Authenticator>::CreateInstance(*r);
        m_h235Authenticators->Append(Auth);
 	}
-#endif
-
 }
 
 GkClient::~GkClient()
 {
-#ifdef OpenH323Factory
     delete m_h235Authenticators;
-#endif
 #ifdef HAS_H46023
 	m_natstrategy.clear();
 #endif
@@ -2289,7 +2271,6 @@ bool GkClient::GetAltGK()
 		// increase our resent time to avoid flooding the parent
 		m_timer = (m_resend *= 2) * 1000;
 		if (m_resend >= m_gkfailtime) {
-			// TODO: not thread-safe ?
 			Toolkit::Instance()->GetRouteTable()->InitTable();
 			m_resend = m_gkfailtime;
 		}
@@ -2488,9 +2469,8 @@ void GkClient::OnRCF(RasMsg *ras)
 		m_gatekeeperId = rcf.m_gatekeeperIdentifier;
 		if (rcf.HasOptionalField(H225_RegistrationConfirm::e_alternateGatekeeper))
 			m_gkList->Set(rcf.m_alternateGatekeeper);
-		if ((m_endpointType == EndpointType_Gateway) &&
-			rcf.HasOptionalField(H225_RegistrationConfirm::e_supportsAdditiveRegistration) && 
-			Toolkit::AsBool(GkConfig()->GetString(EndpointSection, "EnableAdditiveRegistration", "0")))
+		if (rcf.HasOptionalField(H225_RegistrationConfirm::e_supportsAdditiveRegistration)
+			&& Toolkit::AsBool(GkConfig()->GetString(EndpointSection, "EnableAdditiveRegistration", "0")))
 				m_useAdditiveRegistration = true;
 
 		if (m_useAdditiveRegistration)
@@ -2548,19 +2528,7 @@ void GkClient::RunSTUNTest(const H323TransportAddress & addr)
 	if (m_stunClient)
 		return;
 
-	H323TransportAddress s;
-	PString server = AsString(addr);
-#ifdef P_DNS
-	PStringList SRVs;
-	PStringList x = server.Tokenise(":");
-	PString number = "h323:user@" + x[0];
-	if (PDNS::LookupSRV(number, "_stun._udp.", SRVs))
-		s = H323TransportAddress(SRVs[0]);
-	else
-#endif
-		s = addr;
-
-	m_stunClient = new STUNClient(this, s);
+	m_stunClient = new STUNClient(this, addr);
 }
 
 void GkClient::H46023_RCF(H460_FeatureStd * feat)
@@ -2659,15 +2627,6 @@ CallRec::NatStrategy GkClient::H46023_GetNATStategy(const H225_CallIdentifier &
 	return (CallRec::NatStrategy)((i != m_natstrategy.end()) ? i->second.front().GetNatStrategy() : 0);
 }
 
-void GkClient::H46023_RemoveNATStategy(const H225_CallIdentifier & id)
-{
-	PWaitAndSignal m(m_strategyMutex);
-
-	GkNATSocketMap::iterator i = m_natstrategy.find(id);
-	if (i != m_natstrategy.end())
-		m_natstrategy.erase(i);
-}
-
 void GkClient::H46023_SetSocketPair(const H225_CallIdentifier & id, WORD sessionID, UDPProxySocket * rtp, UDPProxySocket * rtcp)
 {
 	PWaitAndSignal m(m_strategyMutex);
@@ -2756,7 +2715,7 @@ void GkClient::H46023_TypeDetected(int nattype)
 void GkClient::OnRRJ(RasMsg *ras)
 {
 	H225_RegistrationReject & rrj = (*ras)->m_recvRAS;
-	m_rrjReason =  "Reason: " + rrj.m_rejectReason.GetTagName();
+	m_rrjReason = "Reason: " + rrj.m_rejectReason.GetTagName();
 	PTRACE(1, "GKC\tRegistration Rejected: " << rrj.m_rejectReason.GetTagName());
 
 	if (rrj.HasOptionalField(H225_RegistrationReject::e_altGKInfo)) {
@@ -3018,37 +2977,6 @@ void GkClient::RemoveLocalAlias(const H225_ArrayOf_AliasAddress & aliases)
 	m_h323Id = newAliasList;
 }
 
-bool FindH460Descriptor(unsigned feat, H225_ArrayOf_FeatureDescriptor & features, int & location)
-{
-	for (PINDEX i =0; i < features.GetSize(); i++) {
-		H225_GenericIdentifier & id = features[i].m_id;
-		if (id.GetTag() == H225_GenericIdentifier::e_standard) {
-			PASN_Integer & asnInt = id;
-			if (asnInt.GetValue() == feat) {
-				location = i;
-				return TRUE;
-			}
-		}
-	}
-	return false;
-}
-
-void RemoveH460Descriptor(unsigned feat, H225_ArrayOf_FeatureDescriptor & features)
-{
-	for(PINDEX i=0; i < features.GetSize(); i++) {
-		H225_GenericIdentifier & id = features[i].m_id;
-		if (id.GetTag() == H225_GenericIdentifier::e_standard) {
-			PASN_Integer & asnInt = id;
-			if (asnInt.GetValue() == feat) {
-				for(PINDEX j=i+1; j < features.GetSize(); j++)
-					features[j-1] = features[j];
-				features.SetSize(features.GetSize() - 1);
-				return;
-			}	
-		}
-	}
-}
-
 bool GkClient::HandleSetup(SetupMsg & setup, bool fromInternal)
 {
 	RewriteE164(setup, fromInternal);
@@ -3058,7 +2986,7 @@ bool GkClient::HandleSetup(SetupMsg & setup, bool fromInternal)
 		if (setupBody.HasOptionalField(H225_Setup_UUIE::e_supportedFeatures)) {
 #ifdef HAS_H46023
 			H225_ArrayOf_FeatureDescriptor & fs = setupBody.m_supportedFeatures;
-			int location = 0;
+			unsigned location = 0;
 			if (m_registeredH46023 && FindH460Descriptor(24, fs, location)) {
 				H460_Feature feat = H460_Feature(fs[location]);
 				H460_FeatureStd & std24 = (H460_FeatureStd &)feat;
@@ -3072,14 +3000,14 @@ bool GkClient::HandleSetup(SetupMsg & setup, bool fromInternal)
 		}
 	} else {
 #ifdef HAS_H46023
-		int nonce = 0;
-		if (setupBody.HasOptionalField(H225_Setup_UUIE::e_supportedFeatures) && 
-			FindH460Descriptor(24,setupBody.m_supportedFeatures, nonce))
-				RemoveH460Descriptor(24,setupBody.m_supportedFeatures); 
+		unsigned nonce = 0;
+		if (setupBody.HasOptionalField(H225_Setup_UUIE::e_supportedFeatures)
+			&& FindH460Descriptor(24,setupBody.m_supportedFeatures, nonce))
+			RemoveH460Descriptor(24,setupBody.m_supportedFeatures); 
 
 		if (m_registeredH46023) {
 			CallRec::NatStrategy natoffload = H46023_GetNATStategy(setupBody.m_callIdentifier);
-			H460_FeatureStd feat = H460_FeatureStd(24); 
+			H460_FeatureStd feat = H460_FeatureStd(24);
 			feat.Add(Std24_NATInstruct,H460_FeatureContent((unsigned)natoffload,8));
 			H225_ArrayOf_FeatureDescriptor & desc = setupBody.m_supportedFeatures;
 			int sz = desc.GetSize();
