@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKSTATUS_H
-#define GKSTATUS_H "@(#) $Id: GkStatus.h,v 1.43 2012/08/15 07:08:13 willamowius Exp $"
+#define GKSTATUS_H "@(#) $Id: GkStatus.h,v 1.44 2013/07/31 16:11:07 willamowius Exp $"
 
 #include <map>
 #include "yasocket.h"
@@ -82,7 +82,7 @@ public:
 
 	/** Notification form the status client that it has been deleted
 	*/
-	void StatusClientDeleted() { --m_statusClients; }
+	void StatusClientDeleted() { if (m_statusClients > 0) --m_statusClients; }
 
 	/** Print a list of all connected status interface users
 		to the requesting client.
