@@ -160,7 +160,7 @@ PrefixInfo LRQSender<R>::operator()(Neighbor *nb, WORD seqnum) const
 	// select neighbor based on dialed IP
 	if (const H225_TransportAddress *dest = m_r.GetDestIP()) {
 		H225_ArrayOf_AliasAddress aliases;
-		if (PrefixInfo info = nb->GetIPInfo(*dest, aliases)) {
+		if (nb->GetIPInfo(*dest, aliases)) {
 			H225_RasMessage lrq_ras;
 			H225_LocationRequest & lrq = nb->BuildLRQ(lrq_ras, seqnum, aliases);
 			if (nb->OnSendingLRQ(lrq, m_r) && nb->SendLRQ(lrq_ras))
@@ -216,6 +216,7 @@ Neighbor::Neighbor()
 	m_keepAliveTimerInterval = 0;
 	m_H46018Server = false;
 	m_H46018Client = false;
+	m_useTLS = false;
 }
 
 Neighbor::~Neighbor()
@@ -363,9 +364,10 @@ bool Neighbor::SetProfile(const PString & id, const PString & type)
 		m_H46018Client = true;
 		SetH46018GkKeepAliveInterval(29);	// start with every 29 seconds
 	}
-	if (Toolkit::AsBool(config->GetString(section, "H46018Server", "0"))) {
-		m_H46018Server = true;
-	}
+	m_H46018Server = Toolkit::AsBool(config->GetString(section, "H46018Server", "0"));
+#endif
+#ifdef HAS_TLS
+	m_useTLS = Toolkit::AsBool(config->GetString(section, "UseTLS", "0"));
 #endif
 
 	SetForwardedInfo(section);
@@ -748,7 +750,7 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const AdmissionRequest & re
         lrq.m_genericData[sz] = std24;
     }
 #endif
-#if 0   // Disable until we workout moving non-standard H.460 stuff to new compile directive. - SH
+#ifdef HAS_H460VEN
      /// OID9  'Remote endpoint vendor info THIS IS "1.3.6.1.4.1.17090.0.9" NOT H.460.9	- SH
      H460_FeatureOID foid9 = H460_FeatureOID(OID9);	 
      int sz = lrq.m_genericData.GetSize();	 
@@ -1029,6 +1031,7 @@ public:
 	bool IsTraversalServer() const { return m_h46018_server; }
 	bool IsTraversalZone() const { return m_h46018_client || m_h46018_server; }
 	bool IsH46024Supported() const;
+	bool UseTLS() const { return m_useTLS; }
 
 	// override from class RasRequester
 	virtual bool IsExpected(const RasMsg *) const;
@@ -1052,9 +1055,10 @@ private:
 	RasMsg *m_result;
 	PString m_neighbor_used;
 	bool m_h46018_client, m_h46018_server;
+	bool m_useTLS;
 };
 
-LRQRequester::LRQRequester(const LRQFunctor & fun) : m_sendto(fun), m_result(0), m_h46018_client(false), m_h46018_server(false)
+LRQRequester::LRQRequester(const LRQFunctor & fun) : m_sendto(fun), m_result(0), m_h46018_client(false), m_h46018_server(false), m_useTLS(false)
 {
 	AddFilter(H225_RasMessage::e_locationConfirm);
 	AddFilter(H225_RasMessage::e_locationReject);
@@ -1091,7 +1095,7 @@ bool LRQRequester::Send(Neighbor * nb)
 	PWaitAndSignal lock(m_rmutex);
 
 	if (PrefixInfo info = m_sendto(nb, m_seqNum))
-		   m_requests.insert(make_pair(info, nb));
+		m_requests.insert(make_pair(info, nb));
 
 	if (m_requests.empty()) {
 		PTRACE(2, "SRV\tError sending LRQ to " << nb->GetIP());
@@ -1131,12 +1135,12 @@ bool LRQRequester::IsH46024Supported() const
 	return false;
 }
 
-bool LRQRequester::IsExpected(const RasMsg *ras) const
+bool LRQRequester::IsExpected(const RasMsg * ras) const
 {
 	return RasHandler::IsExpected(ras) && (ras->GetSeqNum() == m_seqNum);
 }
 
-void LRQRequester::Process(RasMsg *ras)
+void LRQRequester::Process(RasMsg * ras)
 {
 	PWaitAndSignal lock(m_rmutex);
 	for (Queue::iterator iter = m_requests.begin(); iter != m_requests.end(); ++iter) {
@@ -1172,6 +1176,9 @@ void LRQRequester::Process(RasMsg *ras)
 					m_result = ras;
 				AddReply(req.m_reply = ras);
 				m_neighbor_used = req.m_neighbor->GetId(); // record neighbor used
+#ifdef HAS_TLS
+				m_useTLS = req.m_neighbor->UseTLS();
+#endif
 				m_h46018_client = req.m_neighbor->IsH46018Client();
 				m_h46018_server = req.m_neighbor->IsH46018Server();
 				if (m_h46018_server) {
@@ -1201,7 +1208,6 @@ void LRQRequester::Process(RasMsg *ras)
 
 	PTRACE(1, "RAS\tUnknown reply " << ras->GetTagName());
 	delete ras;
-	ras = NULL;
 }
 
 bool LRQRequester::OnTimeout()
@@ -1352,6 +1358,17 @@ PString NeighborList::GetNeighborGkIdBySigAdr(const PIPSocket::Address & sigAd)
 	return (*findNeighbor)->GetGkId();
 }
 
+bool NeighborList::GetNeighborTLSBySigAdr(const PIPSocket::Address & sigAd)
+{
+	// Attempt to find the neigbor in the list
+	List::iterator findNeighbor = find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsFrom), &sigAd));
+	if (findNeighbor == m_neighbors.end())
+	{
+		return false;
+	}
+	return (*findNeighbor)->UseTLS();
+}
+
 PString NeighborList::GetNeighborGkIdBySigAdr(const H225_TransportAddress & sigAd)
 {
 	PIPSocket::Address ipaddr;
@@ -1364,6 +1381,18 @@ PString NeighborList::GetNeighborGkIdBySigAdr(const H225_TransportAddress & sigA
 	return GetNeighborGkIdBySigAdr(ipaddr);
 }
 
+bool NeighborList::GetNeighborTLSBySigAdr(const H225_TransportAddress & sigAd)
+{
+	PIPSocket::Address ipaddr;
+ 
+	// Get the Neigbor IP address from the transport address
+	if (!GetIPFromTransportAddr(sigAd, ipaddr))
+	{
+		return false;
+	}
+	return GetNeighborTLSBySigAdr(ipaddr);
+}
+
 /* Not used currently
 H225_CryptoH323Token BuildAccessToken(const H225_TransportAddress & dest, const PIPSocket::Address & addr)
 {
@@ -1483,8 +1512,8 @@ bool NeighborPolicy::OnRequest(AdmissionRequest & arq_obj)
 	if (request.Send(m_neighbors)) {
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
-#ifdef HAS_H460
-			if ((request.IsTraversalZone() || request.IsH46024Supported()) && RasServer::Instance()->IsGKRouted()) {
+#if defined(HAS_H460) || defined (HAS_TLS)
+			if ((request.UseTLS() || request.IsTraversalZone() || request.IsH46024Supported()) && RasServer::Instance()->IsGKRouted()) {
 				// overwrite callSignalAddress and replace with ours, we must proxy this call
 				endptr ep = RegistrationTable::Instance()->FindByEndpointId(arq_obj.GetRequest().m_endpointIdentifier); // should not be null
 				if (ep) {
@@ -1498,6 +1527,9 @@ bool NeighborPolicy::OnRequest(AdmissionRequest & arq_obj)
 				H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
 				con = *lcf;
 				route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
+				// set flag to use TLS
+				if (request.UseTLS())
+					route.m_destEndpoint->SetUseTLS(true);
 				// set flag to use H.460.18 if neighbor is traversal server
 				if (request.IsTraversalClient()) {
 					// if we are the client, then the call goes to a traversal server
@@ -1597,14 +1629,17 @@ bool NeighborPolicy::OnRequest(LocationRequest & lrq_obj)
 		if (request.Send(m_neighbors, requester)) {
 			if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 				Route route(m_name, lcf->m_callSignalAddress);
-#ifdef HAS_H460
-				if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData) || request.IsTraversalZone()) {
+#if defined(HAS_H460) || defined (HAS_TLS)
+				if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData) || request.IsTraversalZone() || request.UseTLS()) {
 					// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
 					H225_RasMessage ras;
 					ras.SetTag(H225_RasMessage::e_locationConfirm);
 					H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
 					con = *lcf;
 					route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
+					// set flag to use TLS
+					if (request.UseTLS())
+						route.m_destEndpoint->SetUseTLS(true);
 					// set flag to use H.460.18 if neighbor is traversal server
 					if (request.IsTraversalClient()) {
 						// if we are the client, then the call goes to a traversal server
@@ -1644,14 +1679,17 @@ bool NeighborPolicy::OnRequest(SetupRequest & setup_obj)
 	if (request.Send(m_neighbors)) {
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
-#ifdef HAS_H460
-			if (request.IsTraversalZone() || request.IsH46024Supported()) {
+#if defined(HAS_H460) || defined (HAS_TLS)
+			if (request.UseTLS() || request.IsTraversalZone() || request.IsH46024Supported()) {
 				// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
 				H225_RasMessage ras;
 				ras.SetTag(H225_RasMessage::e_locationConfirm);
 				H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
 				con = *lcf;
 				route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
+				// set flag to use TLS
+				if (request.UseTLS())
+					route.m_destEndpoint->SetUseTLS(true);
 				// set flag to use H.460.18 if neighbor is traversal server
 				if (request.IsTraversalClient()) {
 					// if we are the client, then the call goes to a traversal server
@@ -1686,14 +1724,17 @@ bool NeighborPolicy::OnRequest(FacilityRequest & facility_obj)
 	if (request.Send(m_neighbors)) {
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
-#ifdef HAS_H460
-			if (request.IsTraversalZone() || request.IsH46024Supported()) {
+#if defined(HAS_H460) || defined (HAS_TLS)
+			if (request.UseTLS() || request.IsTraversalZone() || request.IsH46024Supported()) {
 				// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
 				H225_RasMessage ras;
 				ras.SetTag(H225_RasMessage::e_locationConfirm);
 				H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
 				con = *lcf;
 				route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
+				// set flag to use TLS
+				if (request.UseTLS())
+					route.m_destEndpoint->SetUseTLS(true);
 				// set flag to use H.460.18 if neighbor is traversal server
 				if (request.IsTraversalClient()) {
 					// if we are the client, then the call goes to a traversal server
@@ -2161,81 +2202,81 @@ NeighborSqlPolicy::NeighborSqlPolicy()
 
 bool NeighborSqlPolicy::ResolveRoute(RoutingRequest & request, DestinationRoutes & destination)
 {
-		if (!destination.ChangeAliases()) {
-			PTRACE(2,  m_name << "\tNothing to resolve.");
+	if (!destination.ChangeAliases()) {
+		PTRACE(2, m_name << "\tNothing to resolve.");
+		return true;
+	}
+
+	if (destination.RejectCall()) {
+		destination.SetRejectReason(H225_AdmissionRejectReason::e_undefinedReason);
+		return false;
+	}
+
+	PString destinationIp = AsString(destination.GetNewAliases()[0],0);
+	PStringArray adr_parts = SplitIPAndPort(destinationIp,GK_DEF_UNICAST_RAS_PORT);
+	PIPSocket::Address ip;
+	if (!IsIPAddress(adr_parts[0]))
+		PIPSocket::GetHostAddress(adr_parts[0],ip);
+	else
+		ip = adr_parts[0];
+	WORD port = (WORD)(adr_parts[1].AsInteger());
+	H323TransportAddress addr(ip,port);
+
+	if (addr == H323TransportAddress(RasServer::Instance()->GetRasAddress(ip))) {
+		PTRACE(2, m_name << "\tERROR LRQ loop detected.");
+		return false;
+	}
+
+	// Create a gatekeeper object
+	GnuGK * nb = new GnuGK();
+	if (!nb->SetProfile(m_name, addr)) {
+		PTRACE(2, "ROUTING\tERROR setting " << m_name << " profile at " << addr);
+		SNMP_TRAP(11, SNMPError, Configuration, "Error setting " + PString(m_name) + " profile at " + AsString(addr));
+		delete nb;
+		destination.SetRejectReason(H225_AdmissionRejectReason::e_undefinedReason);
+		return false;
+	}
+
+	int m_neighborTimeout = GkConfig()->GetInteger(LRQFeaturesSection, "NeighborTimeout", 5) * 100;
+
+	// Send LRQ to retreive callers signaling address
+	// Caution: we may only use the functor object of the right type and never touch the others!
+	LRQSender<AdmissionRequest> ArqFunctor((AdmissionRequest &)request);
+	LRQSender<SetupRequest> SetupFunctor((SetupRequest &)request);
+	LRQSender<FacilityRequest> FacilityFunctor((FacilityRequest &)request);
+	LRQSender<LocationRequest> LrqFunctor((LocationRequest &)request);
+	LRQRequester * pRequest = NULL;
+	if (dynamic_cast<AdmissionRequest *>(&request)) {
+		pRequest = new LRQRequester(ArqFunctor);
+	} else if (dynamic_cast<SetupRequest *>(&request)) {
+		pRequest = new LRQRequester(SetupFunctor);
+	} else if (dynamic_cast<FacilityRequest *>(&request)) {
+		pRequest = new LRQRequester(FacilityFunctor);
+	} else if (dynamic_cast<LocationRequest *>(&request)) {
+		pRequest = new LRQRequester(LrqFunctor);
+	}
+	if (pRequest && pRequest->Send(nb)) {
+		if (H225_LocationConfirm * lcf = pRequest->WaitForDestination(m_neighborTimeout)) {
+			Route route(m_name, lcf->m_callSignalAddress);
+#ifdef HAS_H460
+			if (pRequest->IsH46024Supported()) {
+				H225_RasMessage ras;
+				ras.SetTag(H225_RasMessage::e_locationConfirm);
+				H225_LocationConfirm & conf = (H225_LocationConfirm &)ras;
+				conf = *lcf;
+				route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
+			}
+#endif
+			destination.AddRoute(route);
+			destination.SetChangedAliases(false);
+			delete pRequest;
+			delete nb;
 			return true;
 		}
-
-		if (destination.RejectCall()) {
-			destination.SetRejectReason(H225_AdmissionRejectReason::e_undefinedReason);
-			return false;
-		}
-
-		PString destinationIp = AsString(destination.GetNewAliases()[0],0);
-		PStringArray adr_parts = SplitIPAndPort(destinationIp,GK_DEF_UNICAST_RAS_PORT);
-		PIPSocket::Address ip;
-		if (!IsIPAddress(adr_parts[0]))
-		  PIPSocket::GetHostAddress(adr_parts[0],ip);
-		 else
-		  ip = adr_parts[0];
-		WORD port = (WORD)(adr_parts[1].AsInteger());
-		H323TransportAddress addr(ip,port);
-
-		if (addr == H323TransportAddress(RasServer::Instance()->GetRasAddress(ip))) {
-			PTRACE(2,  m_name << "\tERROR LRQ loop detected.");
-			return false;
-		}
-
-		// Create a gatekeeper object
-		GnuGK * nb = new GnuGK();
-		if (!nb->SetProfile(m_name, addr)) {
-			PTRACE(2, "ROUTING\tERROR setting " << m_name << " profile at " << addr);
-			SNMP_TRAP(11, SNMPError, Configuration, "Error setting " + PString(m_name) + " profile at " + AsString(addr));
-			delete nb;
-			destination.SetRejectReason(H225_AdmissionRejectReason::e_undefinedReason);
-			return false;
-		}
-
-		int m_neighborTimeout = GkConfig()->GetInteger(LRQFeaturesSection, "NeighborTimeout", 5) * 100;
-
-		// Send LRQ to retreive callers signaling address
-		// Caution: we may only use the functor object of the right type and never touch the others!
-		LRQSender<AdmissionRequest> ArqFunctor((AdmissionRequest &)request);
-		LRQSender<SetupRequest> SetupFunctor((SetupRequest &)request);
-		LRQSender<FacilityRequest> FacilityFunctor((FacilityRequest &)request);
-		LRQSender<LocationRequest> LrqFunctor((LocationRequest &)request);
-		LRQRequester * pRequest = NULL;
-		if (dynamic_cast<AdmissionRequest *>(&request)) {
-			pRequest = new LRQRequester(ArqFunctor);
-		} else if (dynamic_cast<SetupRequest *>(&request)) {
-			pRequest = new LRQRequester(SetupFunctor);
-		} else if (dynamic_cast<FacilityRequest *>(&request)) {
-			pRequest = new LRQRequester(FacilityFunctor);
-		} else if (dynamic_cast<LocationRequest *>(&request)) {
-			pRequest = new LRQRequester(LrqFunctor);
-		}
-		if (pRequest && pRequest->Send(nb)) {
-			if (H225_LocationConfirm * lcf = pRequest->WaitForDestination(m_neighborTimeout)) {
-				Route route(m_name, lcf->m_callSignalAddress);
-#ifdef HAS_H460
-				if (pRequest->IsH46024Supported()) {
-					H225_RasMessage ras;
-					ras.SetTag(H225_RasMessage::e_locationConfirm);
-					H225_LocationConfirm & conf = (H225_LocationConfirm &)ras;
-					conf = *lcf;
-					route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
-				}
-#endif
-				destination.AddRoute(route);
-				destination.SetChangedAliases(false);
-				delete pRequest;
-				delete nb;
-				return true;
-			}
-		}
-		delete pRequest;
-		delete nb;
-		return false;
+	}
+	delete pRequest;
+	delete nb;
+	return false;
 }
 
 namespace {
