@@ -3,7 +3,7 @@
 // ProxyChannel.h
 //
 // Copyright (c) Citron Network Inc. 2001-2003
-// Copyright (c) 2002-2012, Jan Willamowius
+// Copyright (c) 2002-2013, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef PROXYCHANNEL_H
-#define PROXYCHANNEL_H "@(#) $Id: ProxyChannel.h,v 1.135 2013/02/28 23:38:57 willamowius Exp $"
+#define PROXYCHANNEL_H "@(#) $Id: ProxyChannel.h,v 1.155.2.1 2013/09/19 16:48:39 willamowius Exp $"
 
 #include <vector>
 #include <list>
@@ -21,6 +21,12 @@
 #include "yasocket.h"
 #include "RasTbl.h"
 #include "gktimer.h"
+#include "config.h"
+
+#ifdef HAS_H46026
+	#include <h460/h46026.h>
+	#include <h460/h46026mgr.h>
+#endif
 
 
 class Q931;
@@ -53,7 +59,12 @@ typedef H225SignalingMsg<H225_Setup_UUIE> SetupMsg;
 typedef H225SignalingMsg<H225_Facility_UUIE> FacilityMsg;
 struct SetupAuthData;
 
+#ifdef _WIN32
+typedef int ssize_t;
+#endif
+
 extern const char *RoutedSec;
+extern const char *TLSSec;
 extern const char *ProxySection;
 
 const WORD DEFAULT_PACKET_BUFFER_SIZE = 2048;
@@ -64,6 +75,10 @@ bool GetUUIE(const Q931 & q931, H225_H323_UserInformation & uuie);
 
 void SetUUIE(Q931 & q931, const H225_H323_UserInformation & uuie);
 
+ssize_t UDPSendWithSourceIP(int fd, void * data, size_t len, const H323TransportAddress & toAddress);
+ssize_t UDPSendWithSourceIP(int fd, void * data, size_t len, const PIPSocket::Address & ip, WORD port);
+
+
 class ProxySocket : public USocket {
 public:
 	enum Result {
@@ -151,8 +166,10 @@ protected:
 	TCPProxySocket *remote;
 	PBYTEArray buffer;
 
-private:
+public:
 	bool InternalWrite(const PBYTEArray &);
+
+protected:
 	bool SetMinBufSize(WORD);
 
 	BYTE *bufptr;
@@ -278,6 +295,9 @@ public:
 	virtual bool EndSession();
 	virtual void OnError();
 
+	// override from TCPProxySocket
+	virtual bool ForwardData();
+
 	void SendReleaseComplete(const H225_CallTerminationCause * = NULL);
 	void SendReleaseComplete(H225_ReleaseCompleteReason::Choices);
 
@@ -285,6 +305,7 @@ public:
 	bool IsH245TunnelingTranslation() const { return m_h245TunnelingTranslation; }
 	PASN_OctetString * GetNextQueuedH245Message();
 	bool HandleH245Mesg(PPER_Stream &, bool & suppress, H245Socket * h245sock = NULL);
+	void SendPostDialDigits();
 	void OnH245ChannelClosed() { m_h245socket = NULL; }
 	Address GetLocalAddr() { return localAddr; }
 	Address GetPeerAddr() { return peerAddr; }
@@ -309,11 +330,14 @@ public:
 	void RemoveH245Handler();
 	void SaveTCS(const H245_TerminalCapabilitySet & tcs) { m_savedTCS = tcs; }
 	H245_TerminalCapabilitySet GetSavedTCS() const { return m_savedTCS; }
-	bool SendTunneledH245(const H245_MultimediaSystemControlMessage & h245msg);
 	bool SendTunneledH245(const PPER_Stream & strm);
+	bool SendTunneledH245(const H245_MultimediaSystemControlMessage & h245msg);
 #ifdef HAS_H235_MEDIA
+	bool IsH245Master() const { return m_isH245Master; }
     bool HandleH235TCS(H245_TerminalCapabilitySet & tcs);
     bool HandleH235OLC(H245_OpenLogicalChannel & olc);
+	void SendEncryptionUpdateCommand(WORD flcn, BYTE oldPT, BYTE plainPT);
+	void SendEncryptionUpdateRequest(WORD flcn, BYTE oldPT, BYTE plainPT);
 #endif
 	H245Socket * GetH245Socket() const { return m_h245socket; }
 	void SetH245Socket(H245Socket * sock) { m_h245socket = sock; }
@@ -327,6 +351,10 @@ public:
 #ifdef HAS_H46017
 	bool SendH46017Message(const H225_RasMessage & ras);
 	void CleanupCall();
+#endif
+#ifdef HAS_H46026
+	bool SendH46026RTP(unsigned sessionID, bool isRTP, const void * data, unsigned len);
+	void PollPriorityQueue();
 #endif
 	bool MaintainConnection() const { return m_maintainConnection; }
 
@@ -344,18 +372,10 @@ public:
 	void LockH245Handler() { m_h245handlerLock.Wait(); }
 	void UnlockH245Handler() { m_h245handlerLock.Signal(); }
 
-#ifdef HAS_H46018
-	bool IsH46019ClientCall(const H225_Setup_UUIE & setupBody);
-#endif
-
 #ifdef HAS_H46023
 	bool IsH46024Call(const H225_Setup_UUIE & setupBody);
 #endif
 
-//#ifdef HAS_H46026
-//	bool IsH46026Call(const H225_Setup_UUIE & setupBody);
-//#endif
-
 	CallSignalSocket * GetRemote() const { return dynamic_cast<CallSignalSocket *>(remote); }
 
 protected:
@@ -412,6 +432,7 @@ private:
 	void BuildReleasePDU(Q931 &, const H225_CallTerminationCause *) const;
 	// if return false, the h245Address field will be removed
 	bool SetH245Address(H225_TransportAddress &);
+
 	bool InternalConnectTo();
 	bool ForwardCallConnectTo();
 
@@ -477,6 +498,7 @@ private:
 	H245_TerminalCapabilitySet m_savedTCS;	// saved tcs to re-send
 #ifdef HAS_H46017
 	bool m_h46017Enabled;
+	TCPProxySocket * rc_remote; // copy of the remote pointer that may be only used to send RC on call end
 #endif
 #ifdef HAS_H46018
 	bool m_callFromTraversalServer; // is this call from a traversal server ?
@@ -486,6 +508,9 @@ private:
 #ifdef HAS_H235_MEDIA
 	bool m_isH245Master;
 #endif
+#ifdef HAS_H46026
+	H46026ChannelManager * m_h46026PriorityQueue;
+#endif
 };
 
 class CallSignalListener : public TCPListenSocket {
@@ -503,6 +528,39 @@ protected:
 	Address m_addr;
 };
 
+
+#ifdef HAS_TLS
+
+class TLSCallSignalListener : public CallSignalListener {
+public:
+	TLSCallSignalListener(const Address &, WORD);
+	~TLSCallSignalListener();
+
+	// override from class CallSignalListener
+	virtual ServerSocket *CreateAcceptor() const;
+};
+
+class TLSCallSignalSocket : public CallSignalSocket {
+public:
+	TLSCallSignalSocket();
+	TLSCallSignalSocket(CallSignalSocket * s, WORD port);
+	virtual ~TLSCallSignalSocket();
+
+	virtual bool Connect(const Address & addr);
+	virtual PBoolean Connect(const Address & iface, WORD localPort, const Address & addr);	// override from TCPProxySocket
+	virtual bool Read(void * buf, int sz);
+	virtual int GetLastReadCount() const { return m_lastReadCount; }
+	virtual bool Write(const void * buf, int sz);
+	virtual void Dispatch();
+
+protected:
+	SSL * m_ssl;
+	int m_lastReadCount;
+};
+
+#endif // HAS_TLS
+
+
 #ifdef HAS_H46018
 
 class MultiplexRTPListener : public UDPSocket {
@@ -534,7 +592,7 @@ public:
 
 	void Dump() const;
 
-	bool IsValid() const { return m_session != INVALID_RTP_SESSION; }
+	bool IsValid() const { return (m_session != INVALID_RTP_SESSION) || (m_flcn > 0); }
 	bool sideAReady(bool isRTCP) const { return isRTCP ? IsSet(m_addrA_RTCP) : IsSet(m_addrA); }
 	bool sideBReady(bool isRTCP) const { return isRTCP ? IsSet(m_addrB_RTCP) : IsSet(m_addrB); }
 	H46019Session SwapSides() const; // return a copy with side A and B swapped
@@ -547,6 +605,7 @@ public:
 //protected:
 	H225_CallIdentifier m_callid;
 	WORD m_session;
+	WORD m_flcn;		// only used to assign master assigned RTP session IDs
 	void * m_openedBy;	// side A (pointer to H245ProxyHandler used as an ID)
 	void * m_otherSide;	// side B (pointer to H245ProxyHandler used as an ID)
 	H323TransportAddress m_addrA;
@@ -595,6 +654,7 @@ public:
 	virtual void OnReload() { /* currently not runtime changable */ }
 
 	virtual void AddChannel(const H46019Session & cha);
+	virtual void UpdateChannelSession(const H225_CallIdentifier & callid, WORD flcn, void * openedBy, WORD session);
 	virtual void UpdateChannel(const H46019Session & cha);
 	virtual H46019Session GetChannel(const H225_CallIdentifier & callid, WORD session) const;
 	virtual H46019Session GetChannelSwapped(const H225_CallIdentifier & callid, WORD session, void * openedBy) const;
@@ -604,7 +664,10 @@ public:
 #endif
 	virtual void DumpChannels(const PString & msg = "") const;
 
-	virtual void HandlePacket(PUInt32b receivedMultiplexID, const H323TransportAddress & fromAddress, void * data, unsigned len, bool isRTCP);
+	virtual bool HandlePacket(PUInt32b receivedMultiplexID, const H323TransportAddress & fromAddress, void * data, unsigned len, bool isRTCP);
+#ifdef HAS_H46026
+	virtual bool HandlePacket(const H225_CallIdentifier & callid, const H46026_UDPFrame & data);
+#endif
 
 	virtual int GetRTPOSSocket() const { return m_reader ? m_reader->GetRTPOSSocket() : INVALID_OSSOCKET; }
 	virtual int GetRTCPOSSocket() const { return m_reader ? m_reader->GetRTCPOSSocket() : INVALID_OSSOCKET; }
@@ -620,6 +683,66 @@ protected:
 };
 #endif
 
+
+#ifdef HAS_H46026
+
+// class for a H.460.26 session: it includes only the plain RTP side (multiplexed RTP is handled in H46019Session
+class H46026Session
+{
+public:
+	H46026Session() : m_isValid(false) { }
+	H46026Session(const H225_CallIdentifier & callid, WORD session, int osRTPSocket, int osRTCPSocket,
+					const H323TransportAddress & toRTP, const H323TransportAddress & toRTCP);
+
+	void Send(void * data, unsigned len, bool isRTCP);
+	bool IsValid() const { return m_isValid; }
+	void Dump() const;
+
+//protected:
+	bool m_isValid;
+	H225_CallIdentifier m_callid;
+	WORD m_session;
+	int m_osRTPSocket;
+	int m_osRTCPSocket;
+	H323TransportAddress m_toAddressRTP;
+	H323TransportAddress m_toAddressRTCP;
+
+#ifdef HAS_H235_MEDIA
+	RTPLogicalChannel * m_encryptingLC;
+	RTPLogicalChannel * m_decryptingLC;
+#endif
+};
+
+// handles stores H.460.26 RTP sessions
+class H46026RTPHandler : public Singleton<H46026RTPHandler> {
+public:
+	H46026RTPHandler();
+	virtual ~H46026RTPHandler();
+
+	virtual void OnReload() { /* currently not runtime changable */ }
+
+	virtual void AddChannel(const H46026Session & chan);
+	virtual void ReplaceChannel(const H46026Session & chan);
+	virtual void UpdateChannelRTP(const H225_CallIdentifier & callid, WORD session, H323TransportAddress toRTP);
+	virtual void UpdateChannelRTCP(const H225_CallIdentifier & callid, WORD session, H323TransportAddress toRTCP);
+	virtual void RemoveChannels(H225_CallIdentifier callid);	// pass by value in case call gets removed
+	H46026Session FindSession(const H225_CallIdentifier & callid, WORD session) const;
+#ifdef HAS_H235_MEDIA
+	virtual void UpdateChannelEncryptingLC(const H225_CallIdentifier & callid, WORD session, RTPLogicalChannel * lc);
+	virtual void UpdateChannelDecryptingLC(const H225_CallIdentifier & callid, WORD session, RTPLogicalChannel * lc);
+#endif
+	virtual void DumpChannels(const PString & msg = "") const;
+
+	virtual bool HandlePacket(const H225_CallIdentifier & callid, H46026_UDPFrame & data);
+
+protected:
+	mutable PReadWriteMutex m_listLock;
+	list<H46026Session> m_h46026channels;
+};
+
+#endif
+
+
 class ProxyHandler : public SocketsReader {
 public:
 	ProxyHandler(const PString& name);
