@@ -2,7 +2,7 @@
 //
 // bookkeeping for RAS-Server in H.323 gatekeeper
 //
-// Copyright (c) 2000-2012, Jan Willamowius
+// Copyright (c) 2000-2013, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASTBL_H
-#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.265 2013/05/01 19:35:46 shorne Exp $"
+#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.277.2.1 2013/09/26 22:37:13 willamowius Exp $"
 
 #include <list>
 #include <map>
@@ -129,9 +129,8 @@ public:
 
 	virtual void SetRasAddress(const H225_TransportAddress &);
 	virtual void SetCallSignalAddress(const H225_TransportAddress &);
-	virtual void SetEndpointIdentifier(const H225_EndpointIdentifier &);
 	virtual void SetTimeToLive(int);
-	virtual void SetAliases(const H225_ArrayOf_AliasAddress &, PBoolean = false);
+	virtual bool SetAliases(const H225_ArrayOf_AliasAddress &, PBoolean = false);
 	virtual bool RemoveAliases(const H225_ArrayOf_AliasAddress &);
 	virtual void AddNumbers(const PString & numbers);
 	virtual void SetEndpointType(const H225_EndpointType &);
@@ -143,8 +142,8 @@ public:
 	virtual void Update(const H225_RasMessage & lightweightRRQ);
 	virtual bool IsGateway() const { return false; }
 
-	virtual void SetAdditiveRegistrant() {};
-	virtual bool IsAdditiveRegistrant() const { return false; }
+	virtual void SetAdditiveRegistrant();
+	virtual bool IsAdditiveRegistrant() const;
 
 	/** Find if one of the given aliases matches any alias for this endpoint.
 
@@ -244,7 +243,7 @@ public:
 	bool HasPresenceData();
 #ifdef HAS_H460P
 	void ParsePresencePDU(const PASN_OctetString & pdu);
-#if H460P_VER < 3
+#ifndef HAS_H460P_VER_3
 	bool BuildPresencePDU(unsigned msgtag, PASN_OctetString & pdu);
 #endif
 #endif
@@ -309,16 +308,22 @@ public:
 	PString GetAdditionalDestinationAlias() const { return m_additionalDestAlias; }
 
 	bool IsH46017Disabled() const { return m_h46017disabled; }
-	void SetUsesH46017(bool val) {  m_usesH46017 = val; }
+	void SetUsesH46017(bool val) { m_usesH46017 = val; }
 	bool UsesH46017() const { return m_usesH46017; }
 	bool IsH46018Disabled() const { return m_h46018disabled; }
-	void SetUsesH46026(bool val) {  m_usesH46026 = val; }
+	void SetUsesH46026(bool val) { m_usesH46026 = val; }
 	bool UsesH46026() const { return m_usesH46026; }
+#ifdef HAS_H46026
+	unsigned GetH46026BW() const { return (m_maxBandwidth > 0) ? (m_maxBandwidth / 2) : 384000; }
+#endif
 	void SetTraversalRole(H46019TraversalType val) { m_traversalType = val; m_nat = (val == TraversalClient); }
 	H46019TraversalType GetTraversalRole() const { return m_traversalType; }
 	bool IsTraversalServer() const { return m_traversalType == TraversalServer; }
 	bool IsTraversalClient() const { return m_traversalType == TraversalClient; }
 
+	void SetUseTLS(bool val) { m_useTLS = val; }
+	bool UseTLS() const { return m_useTLS; }
+
 	// smart pointer for EndpointRec
 	typedef SmartPtr<EndpointRec> Ptr;
 
@@ -405,6 +410,8 @@ protected:
 	
 	long m_bandwidth;	// bandwidth currently occupied by this endpoint
 	long m_maxBandwidth; // maximum bandwidth allowed for this endpoint
+	bool m_useTLS;
+	bool m_additiveRegistrant;
 };
 
 typedef EndpointRec::Ptr endptr;
@@ -422,9 +429,6 @@ public:
 	virtual void Update(const H225_RasMessage & lightweightRRQ);
 	virtual bool IsGateway() const { return true; }
 
-	virtual void SetAdditiveRegistrant();
-	virtual bool IsAdditiveRegistrant() const;
-
 	/// Overiden from EndpointRec
 	virtual bool LoadConfig();
 
@@ -488,8 +492,6 @@ protected:
 	bool defaultGW;
 	/// priority for this gateway (when more than one gw matches a dialed number)
 	int priority;
-
-	bool additiveRegistrant;
 };
 
 
@@ -647,11 +649,7 @@ public:
 
 	unsigned flcn;
 	KeepAliveType type;
-#ifdef hasIPV6
-	sockaddr_in6 dest;
-#else
-	sockaddr_in dest;
-#endif
+	H323TransportAddress dest;
 	unsigned interval;
 	unsigned seq;
 	int ossocket;
@@ -752,6 +750,10 @@ public:
 	endptr GetForwarder() const { return m_Forwarder; }
 	long GetBandwidth() const { return m_bandwidth; }
 
+	bool HasPostDialDigits() const { return !m_postdialdigits.IsEmpty(); }
+	PString GetPostDialDigits() const { return m_postdialdigits; }
+	void SetPostDialDigits(const PString & digits) { m_postdialdigits = digits; }
+
 	/** @return
 	    A bit mask with NAT flags for calling and called parties. 
 	    See #NATType enum# for more details.
@@ -1267,6 +1269,12 @@ public:
 		const PString & number /// Dialed-Number
 		);
 
+	// vendor info for calling and called party
+	void SetCallingVendor(const PString & vendor, const PString & version);
+	void GetCallingVendor(PString & vendor, PString & version) const;
+	void SetCalledVendor(const PString & vendor, const PString & version);
+	void GetCalledVendor(PString & vendor, PString & version) const;
+
 	/** @return
 	    Fixed destination address for the call (size 0 if not set).
 	*/
@@ -1311,7 +1319,7 @@ public:
 	PBYTEArray GetRADIUSClass() const;
 	
 	bool IsProceedingSent() const { return m_proceedingSent; }
-	void SetProceedingSent(bool val) { m_proceedingSent = val; }
+	void SetProceedingSent(bool val) { m_proceedingSent = m_proceedingSent || val; }
 
 	void SetRerouteState(RerouteState state) { m_rerouteState = state; }
 	RerouteState GetRerouteState() const { return m_rerouteState; }
@@ -1387,6 +1395,8 @@ private:
 	PString m_destInfo;
 	/// bandwidth occupied by this call (as declared in ARQ)
 	long m_bandwidth;
+	/// post dial digits (can be any alphanumeric user input)
+	PString m_postdialdigits;
 
 	PString m_callerAddr, m_callerId;
 	PString m_calleeAddr, m_calleeId;
@@ -1575,6 +1585,11 @@ private:
 	PMutex m_PTMutex;
     BYTE m_dynamicPayloadTypeCounter;
 #endif
+	// vendor info for calling and called party - only used if no EndpointRec available!
+	PString m_callingVendor;
+	PString m_callingVersion;
+	PString m_calledVendor;
+	PString m_calledVersion;
 };
 
 typedef CallRec::Ptr callptr;
@@ -2116,7 +2131,7 @@ public:
 	H225_CallIdentifier GetCallIdentifier() const { return m_callid; }
 	unsigned GetCallRef() const { return m_callref; }
 	bool IsProceedingSent() const { return m_proceedingSent; }
-	void SetProceedingSent(bool val) { m_proceedingSent = val; }
+	void SetProceedingSent(bool val) { m_proceedingSent = m_proceedingSent || val; }
 
 private:
 	CallSignalSocket * m_socket;
