@@ -129,7 +129,7 @@ bool RoutingRequest::AddRoute(const Route & route)
 	WORD port;
 	if (!(route.m_destAddr.IsValid() && GetIPAndPortFromTransportAddr(route.m_destAddr, addr, port) 
 			&& addr.IsValid() && port != 0)) {
-		PTRACE(1, "ROUTING\tInvalid destination address: " << route.m_destAddr);
+		PTRACE(1, "ROUTING\tInvalid destination address: " << AsString(route.m_destAddr));
 		return false;
 	}
 	list<Route>::const_iterator i = m_failedRoutes.begin();
@@ -160,6 +160,16 @@ void RoutingRequest::RemoveAllRoutes()
 	m_routes.clear();
 }
 
+bool RoutingRequest::GetGatewayDestination(H225_TransportAddress & gw ) const 
+{
+	PIPSocket::Address addr;
+	if (!GetIPFromTransportAddr(m_gwDestination, addr)||!addr.IsValid())
+			return false;
+	
+	gw = m_gwDestination;
+	return true;
+}
+
 // class AdmissionRequest
 template<> H225_ArrayOf_AliasAddress *AdmissionRequest::GetAliases()
 {
@@ -254,8 +264,8 @@ void Policy::SetInstance(const PString & instance)
 {
 	PString policyName = PString(m_name); 
 	if (!instance.IsEmpty()) {
-		policyName = PString(m_name) + "::" + instance;
-		m_name = policyName;
+		policyName = policyName + "::" + instance;
+		m_name = *(PString *)policyName.Clone();
 	}
 	m_iniSection = "Routing::" + policyName;
 
@@ -296,7 +306,7 @@ void Analyzer::OnReload()
 				prefix = "*";
 			PStringArray prefixes(prefix.Tokenise(",;|", false));
 			for (PINDEX k = 0; k < prefixes.GetSize(); ++k)
-				rules[prefixes[k]] = Create(cfgs.GetDataAt(j));
+				rules[prefixes[k]] = Create(cfgs.GetDataAt(j).Trim());
 			PTRACE(1, SectionName[i] << " add policy " << cfgs.GetDataAt(j) << " for prefix " << prefix);
 		}
 		// default policy for backward compatibility
@@ -771,7 +781,13 @@ bool DNSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 		WORD port = (WORD)parts[1].AsUnsigned();
 		PIPSocket::Address addr;
 		if (DNSLookup(domain, addr) && addr.IsValid()) {
-			H225_TransportAddress dest = SocketToH225TransportAddr(addr, port);
+			H225_TransportAddress dest;
+			if (!request.GetGatewayDestination(dest)) {
+				dest = SocketToH225TransportAddr(addr, port);
+				H323SetAliasAddress(alias.Left(at), aliases[i]);
+				PTRACE(4, "ROUTING\tDNS policy resolves to " << alias.Left(at) << " @ " << AsDotString(dest));
+			}
+
 			if (Toolkit::Instance()->IsGKHome(addr)) {
 				// check if the domain is my IP, if so route to local endpoint if available
 				H225_ArrayOf_AliasAddress find_aliases;
@@ -784,13 +800,10 @@ bool DNSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 					continue;	// can't route this alias locally, try next alias
 				}
 			}
+			request.SetFlag(RoutingRequest::e_aliasesChanged);
 			Route route(m_name, dest);
 			route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(dest);
 			request.AddRoute(route);
-			// remove the domain name part (should not be necessary with latest GnuGk as destination, but keep for older ones)
-			H323SetAliasAddress(alias.Left(at), aliases[i]);
-			request.SetFlag(RoutingRequest::e_aliasesChanged);
-			PTRACE(4, "ROUTING\tDNS policy resolves to " << alias.Left(at) << " @ " << AsDotString(dest));
 			return true;
 		}
 	}
@@ -886,7 +899,7 @@ void VirtualQueue::OnReload()
 	
 	m_requestTimeout = GkConfig()->GetInteger(
 		CTIsection, 
-		GkConfig()->HasKey(CTIsection,"RequestTimeout")
+		GkConfig()->HasKey(CTIsection, "RequestTimeout")
 			? "RequestTimeout" : "CTI_Timeout", 
 		DEFAULT_ROUTE_REQUEST_TIMEOUT
 		) * 1000;
@@ -1537,7 +1550,7 @@ void ENUMPolicy::LoadConfig(const PString & instance)
 	if (instance.IsEmpty())
 		return;
 
-	m_enum_schema.SetSize(0);
+	m_enum_schema.RemoveAll();
 	m_enum_schema = GkConfig()->GetAllKeyValues(m_iniSection);
 }
 
@@ -1551,30 +1564,40 @@ bool ENUMPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddre
 
 		PBoolean changed = false;
 		for (PINDEX j = 0; j < aliases.GetSize(); ++j) {
-			FindByAliasesInternal(enum_schema, request, aliases, changed);
+			if (!FindByAliasesInternal(enum_schema, request, aliases, changed))
+				continue;
+
 			if (!gwDestination && changed) {   // If we have a gateway destination and changed
+				PStringArray parts = SplitIPAndPort(gwDestination, GK_DEF_ENDPOINT_SIGNAL_PORT);
+				PString dom = parts[0];
+				PIPSocket::Address addr;
+				if (PIPSocket::GetHostAddress(dom, addr)) 
+					dom = addr.AsString();
+				WORD port = (WORD)parts[1].AsUnsigned();
+				H225_TransportAddress dest;
+				if (!GetTransportAddress(dom, port, dest)) {
+					PTRACE(4, "ROUTING\tPolicy " << m_name << " " << enum_schema << " Could not resolve " << gwDestination);
+					return false;
+				}
+
 				PString alias(AsString(aliases[j], FALSE));
 				int at = alias.Find('@');
 				PString domain = alias.Mid(at+1);
 				if (IsIPAddress(domain) 
 					|| (domain.FindRegEx(PRegularExpression(":[0-9]+$", PRegularExpression::Extended)) != P_MAX_INDEX)) {
-						// add a route and stop going any further
-						PStringArray parts = SplitIPAndPort(domain, GK_DEF_ENDPOINT_SIGNAL_PORT);
-						domain = parts[0];
-						WORD port = (WORD)parts[1].AsUnsigned();
-						H225_TransportAddress dest;
-						if (GetTransportAddress(domain, port, dest)) {
-						PTRACE(4, "ROUTING\tPolicy " << m_name << " " << enum_schema << " set destination for " << alias << " to " << dest);
-							Route * route = NULL;
-							route = new Route(m_name, dest);
-							route->m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(dest);
-							request.AddRoute(*route);
-							delete route;
-							return true;
-						}
+					// add a route and stop going any further
+					PTRACE(4, "ROUTING\tPolicy " << m_name << " " << enum_schema << " set destination for " << alias << " to " << AsString(dest));
+					Route * route = new Route(m_name, dest);
+					route->m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(dest);
+					request.AddRoute(*route);
+					delete route;
+					return true;
 				}
-				changed = false;
+				PTRACE(4, "ROUTING\tPolicy " << m_name << " " << enum_schema << " store destination for " << alias << " to " << AsString(dest));
+				request.SetGatewayDestination(dest);
 			}
+			changed = false;
+			return false;
 		}
 	}
 #endif
@@ -1587,7 +1610,10 @@ bool ENUMPolicy::FindByAliasesInternal(const PString & schema, RoutingRequest &
 #if P_DNS
 	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
 		PString alias(AsString(aliases[i], FALSE));
-
+		if (alias.Left(2) *= "00") { // Check if not GDS number  - SH
+			PTRACE(4, "\t" << m_name << " " << schema << " Ignored " << alias << " Not ENUM format.");
+			continue;
+		}
 		// make sure the number has only digits
 		alias.Replace("+","", true);
 		alias.Replace("*","", true);
@@ -1613,6 +1639,7 @@ bool ENUMPolicy::FindByAliasesInternal(const PString & schema, RoutingRequest &
 				request.SetFlag(RoutingRequest::e_aliasesChanged);
 				H323SetAliasAddress(str, aliases[i]);
 				changed = true;
+				return true;
 			}
 		}
 	}
@@ -1691,7 +1718,11 @@ bool DynamicPolicy::OnRequest(AdmissionRequest & request)
 		RunPolicy(	/* in */ source, calledAlias, calledIP, caller, callingStationId, callid, messageType, clientauthid,
 						/* out: */ destination);
 
+		if (destination.m_routes.empty() && !ResolveRoute(request,destination))
+			destination.SetRejectCall(true);
+
 		if (destination.RejectCall()) {
+			destination.SetChangedAliases(false);
 			request.SetFlag(RoutingRequest::e_Reject);
 			request.SetRejectReason(destination.GetRejectReason());
 		}
@@ -1746,7 +1777,11 @@ bool DynamicPolicy::OnRequest(LocationRequest & request)
 	RunPolicy(	/* in */ source, calledAlias, calledIP, caller, callingStationId,callid, messageType, clientauthid,
 					/* out: */ destination);
 
+	if (destination.m_routes.empty() && !ResolveRoute(request,destination))
+		destination.SetRejectCall(true);
+
 	if (destination.RejectCall()) {
+		destination.SetChangedAliases(false);
 		request.SetFlag(RoutingRequest::e_Reject);
 		request.SetRejectReason(destination.GetRejectReason());
 	}
@@ -1794,7 +1829,11 @@ bool DynamicPolicy::OnRequest(SetupRequest & request)
 	RunPolicy(	/* in */ source, calledAlias, calledIP, caller, callingStationId, callid, messageType, clientauthid,
 					/* out: */ destination);
 
+	if (destination.m_routes.empty() && !ResolveRoute(request,destination))
+		destination.SetRejectCall(true);
+
 	if (destination.RejectCall()) {
+		destination.SetChangedAliases(false);
 		request.SetFlag(RoutingRequest::e_Reject);
 		request.SetRejectReason(destination.GetRejectReason());
 	}
@@ -2043,8 +2082,14 @@ bool CatchAllPolicy::CatchAllRoute(RoutingRequest & request) const
 	H323SetAliasAddress(m_catchAllAlias, find_aliases[0]);
 	endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
 	if (ep) {
-		Route route("catchall", ep, 999);
-		request.AddRoute(route);
+		if (ep->GetEndpointType().HasOptionalField(H225_EndpointType::e_gateway)) {
+			Route route(m_name,ep->GetCallSignalAddress(), 999);
+			route.m_destEndpoint = ep;
+			request.AddRoute(route);
+		} else {
+			Route route(m_name, ep, 999);
+			request.AddRoute(route);
+		}
 		return (m_next == NULL);
 	}
 	PTRACE(1, m_name << "\tCatch-all endpoint " << m_catchAllAlias << " not found!");
