@@ -2,7 +2,7 @@
 //
 // Toolkit class for the GnuGk
 //
-// Copyright (c) 2000-2012, Jan Willamowius
+// Copyright (c) 2000-2013, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef TOOLKIT_H
-#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.106 2013/04/17 21:11:50 willamowius Exp $"
+#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.114 2013/09/10 15:49:47 willamowius Exp $"
 
 #include <vector>
 #include <map>
@@ -27,6 +27,16 @@
 #ifdef HAS_H460P
 #include "h460presence.h"
 #endif
+#ifdef HAS_TLS
+extern "C" {
+#include <openssl/err.h>
+#include <openssl/rand.h>
+#include <openssl/ssl.h>
+// on Windows PTLib 2.10.x doesn't set an #pragma include_alias for this file,
+// so the OpenSSL include dir must be added to IDE settings
+#include <openssl/x509v3.h>	// needed for certificate check
+}
+#endif
 
 class H225_AliasAddress;
 class H225_ArrayOf_AliasAddress;
@@ -215,13 +225,13 @@ class Toolkit : public Singleton<Toolkit>
 		RewriteTool() : m_TrailingChar(' '), m_Rewrite(0) {}
 		~RewriteTool() { delete m_Rewrite; }
 		void LoadConfig(PConfig *);
-		bool RewritePString(PString &) const;
+		bool RewritePString(PString & str) const;
 
 	private:
 		PString m_RewriteFastmatch;
 		char m_TrailingChar;
 		PStringArray m_defaultDomain;
-		RewriteData *m_Rewrite;
+		RewriteData * m_Rewrite;
 	};
 
 	class AssignedAliases {
@@ -343,6 +353,12 @@ class Toolkit : public Singleton<Toolkit>
     bool CreateH350Session(H350_Session * session);
 #endif
 
+#ifdef HAS_TLS
+	bool IsTLSEnabled() const;
+	SSL_CTX * GetTLSContext();
+	bool MatchHostCert(SSL * ssl, PIPSocket::Address addr);
+#endif
+
 	/// maybe modifies #alias#. returns true if it did
 	bool RewriteE164(H225_AliasAddress & alias);
 	bool RewriteE164(H225_ArrayOf_AliasAddress & aliases);
@@ -367,7 +383,7 @@ class Toolkit : public Singleton<Toolkit>
 			~GWRewriteTool();
 			void LoadConfig(PConfig *);
 			void PrintData();
-			bool RewritePString(const PString & gw, bool direction, PString & data);
+			bool RewritePString(const PString & gw, bool direction, PString & data, callptr call);
 
 		private:
 			PDictionary<PString, GWRewriteEntry> m_GWRewrite;
@@ -375,9 +391,9 @@ class Toolkit : public Singleton<Toolkit>
 	};
 
 	// Equivalent functions to RewriteE164 group
-	bool GWRewriteE164(const PString & gw, bool direction, H225_AliasAddress &alias);
-	bool GWRewriteE164(const PString & gw, bool direction, H225_ArrayOf_AliasAddress &aliases);
-	bool GWRewritePString(const PString & gw, bool direction, PString & data) { return m_GWRewrite.RewritePString(gw, direction, data); }
+	bool GWRewriteE164(const PString & gw, bool direction, H225_AliasAddress & alias, callptr call = callptr(NULL));
+	bool GWRewriteE164(const PString & gw, bool direction, H225_ArrayOf_AliasAddress & aliases, callptr call = callptr(NULL));
+	bool GWRewritePString(const PString & gw, bool direction, PString & data, callptr call) { return m_GWRewrite.RewritePString(gw, direction, data, call); }
 
 	PString GetGKHome(std::vector<PIPSocket::Address> &) const;
 	void SetGKHome(const PStringArray &);
@@ -387,6 +403,7 @@ class Toolkit : public Singleton<Toolkit>
 	bool RemoveH235TokensFrom(const PIPSocket::Address & addr) const;
 #ifdef HAS_H235_MEDIA
 	bool IsH235HalfCallMediaEnabled() const { return m_H235HalfCallMediaEnabled; }
+	bool IsH235HalfCallMediaKeyUpdatesEnabled() const { return m_H235HalfCallMediaKeyUpdatesEnabled; }
 #endif
 
 	bool isBehindNAT(PIPSocket::Address &) const;
@@ -603,9 +620,7 @@ class Toolkit : public Singleton<Toolkit>
 	bool IsCauseCodeTranslationActive() const { return m_causeCodeTranslationActive; }
 	void SetCauseCodeTranslationActive(bool val) { m_causeCodeTranslationActive = val; }
 
-#ifdef OpenH323Factory
 	PStringList GetAuthenticatorList();
-#endif
 
 protected:
 	void CreateConfig();
@@ -667,6 +682,7 @@ private:
 	std::vector<NetworkAddress> m_removeH235TokensfromNetwork;
 #ifdef HAS_H235_MEDIA
 	bool m_H235HalfCallMediaEnabled;
+	bool m_H235HalfCallMediaKeyUpdatesEnabled;
 #endif
 	// is H460.18 enabled ?
 #ifdef HAS_H46018
@@ -675,7 +691,8 @@ private:
 	// is H460.23 enabled ?
 #ifdef HAS_H46023
 	bool m_H46023Enabled;
-	std::map<int,H323TransportAddress> m_H46023STUN;
+	std::map<int,std::vector<H323TransportAddress> > m_H46023STUN;
+	PMutex m_stunMutex;
 #endif
 
 #ifdef HAS_H460P
@@ -683,6 +700,11 @@ private:
 #endif
 	std::map<PortType, PString> m_portOpenNotifications;
 	std::map<PortType, PString> m_portCloseNotifications;
+
+#ifdef HAS_TLS
+	SSL_CTX * m_sslCtx;
+	PString m_passphrase;
+#endif
 };
 
 
