@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef FACTORY_H
-#define FACTORY_H "@(#) $Id: factory.h,v 1.13 2011/09/18 23:08:53 willamowius Exp $"
+#define FACTORY_H "@(#) $Id: factory.h,v 1.14 2012/04/09 18:01:30 willamowius Exp $"
 
 /*****************************************************************
 //
@@ -273,7 +273,7 @@ Functor<Product *> *Factory<Product, Identifier>::FindCreator(Identifier n)
 	else if (m_default)
 		return m_default;
 	PTRACE(1, "Init\tCan't create unknown class " << n);
-	return 0;
+	return NULL;
 }
 
 template<class Product, typename Identifier>
