@@ -2,7 +2,7 @@
 //
 // gk.cxx for GNU Gatekeeper
 //
-// Copyright (c) 2000-2012, Jan Willamowius
+// Copyright (c) 2000-2013, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -40,6 +40,10 @@ using std::vector;
 
 PCREATE_PROCESS(Gatekeeper)
 
+#if defined(_WIN32) && (_WIN32_WINNT >= WINDOWS_VISTA)
+LPFN_WSASENDMSG g_pfWSASendMsg = NULL;
+#endif
+
 PTextFile* Gatekeeper::m_logFile = NULL;
 PFilePath Gatekeeper::m_logFilename;
 PMutex Gatekeeper::m_logFileMutex;
@@ -127,6 +131,9 @@ const char * KnownConfigEntries[][2] = {
 	{ "Endpoint", "TimeToLive" },
 	{ "Endpoint", "Type" },
 	{ "Endpoint", "UnregisterOnReload" },
+#ifdef HAS_TLS
+	{ "Endpoint", "UseTLS" },
+#endif
 	{ "Endpoint", "UseAlternateGK" },
 	{ "Endpoint", "Vendor" },
 	{ "FileAcct", "CDRString" },
@@ -177,6 +184,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "Gatekeeper::Main", "Home" },
 	{ "Gatekeeper::Main", "ListenQueueLength" },
 	{ "Gatekeeper::Main", "MaxASNArraySize" },
+	{ "Gatekeeper::Main", "MaxSocketQueue" },
 	{ "Gatekeeper::Main", "MaxStatusClients" },
 	{ "Gatekeeper::Main", "MaximumBandwidthPerCall" },
 	{ "Gatekeeper::Main", "MinimumBandwidthPerCall" },
@@ -199,6 +207,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "Gatekeeper::Main", "UnicastRasPort" },
 	{ "Gatekeeper::Main", "UseBroadcastListener" },
 	{ "Gatekeeper::Main", "UseMulticastListener" },
+#ifdef H323_H350
 	{ "GkH350::Settings", "AssignedAliases" },
 	{ "GkH350::Settings", "BindAuthMode" },
 	{ "GkH350::Settings", "BindUserDN" },
@@ -211,6 +220,23 @@ const char * KnownConfigEntries[][2] = {
 #ifdef hasLDAPStartTLS
 	{ "GkH350::Settings", "StartTLS" },
 #endif
+#endif
+#ifdef P_LDAP
+	{ "GkLDAP::LDAPAttributeNames", "CallDestination" },
+	{ "GkLDAP::LDAPAttributeNames", "H235PassWord" },
+	{ "GkLDAP::LDAPAttributeNames", "H323ID" },
+	{ "GkLDAP::LDAPAttributeNames", "IPAddress" },
+	{ "GkLDAP::LDAPAttributeNames", "TelephonNo" },
+	{ "GkLDAP::Settings", "BindAuthMode" },
+	{ "GkLDAP::Settings", "BindUserDN" },
+	{ "GkLDAP::Settings", "BindUserPW" },
+	{ "GkLDAP::Settings", "SearchBaseDN" },
+	{ "GkLDAP::Settings", "ServerName" },
+	{ "GkLDAP::Settings", "ServerPort" },
+	{ "GkLDAP::Settings", "sizelimit" },
+	{ "GkLDAP::Settings", "StartTLS" },
+	{ "GkLDAP::Settings", "timelimit" },
+#endif
 #ifdef HAS_DATABASE
 	{ "GkPresence::SQL", "CacheTimeout" },
 	{ "GkPresence::SQL", "Database" },
@@ -345,7 +371,6 @@ const char * KnownConfigEntries[][2] = {
 	{ "RasSrv::RRQFeatures", "AccHTTPLink" },
 	{ "RasSrv::RRQFeatures", "AliasTypeFilter" },
 	{ "RasSrv::RRQFeatures", "AuthenticatedAliasesOnly" },
-	{ "RasSrv::RRQFeatures", "EnableAdditiveRegistration" },
 	{ "RasSrv::RRQFeatures", "GatewayAssignAliases" },
 	{ "RasSrv::RRQFeatures", "IRQPollCount" },
 	{ "RasSrv::RRQFeatures", "OverwriteEPOnSameAddress" },
@@ -387,6 +412,7 @@ const char * KnownConfigEntries[][2] = {
 	{ "RoutedMode", "ENUMservers" },
 #ifdef HAS_H235_MEDIA
 	{ "RoutedMode", "EnableH235HalfCallMedia" },
+	{ "RoutedMode", "EnableH235HalfCallMediaKeyUpdates" },
 #endif
 	{ "RoutedMode", "EnableH450.2" },
 #ifdef HAS_H46017
@@ -451,11 +477,15 @@ const char * KnownConfigEntries[][2] = {
 	{ "RoutedMode", "SupportCallingNATedEndpoints" },
 	{ "RoutedMode", "SupportNATedEndpoints" },
 	{ "RoutedMode", "TcpKeepAlive" },
+	{ "RoutedMode", "TLSCallSignalPort" },
 	{ "RoutedMode", "TranslateFacility" },
 	{ "RoutedMode", "TranslateReceivedQ931Cause" },
 	{ "RoutedMode", "TreatUnregisteredNAT" },
 	{ "RoutedMode", "TranslateSentQ931Cause" },
 	{ "RoutedMode", "TranslateSorensonSourceInfo" },
+#ifdef HAS_H46026
+	{ "RoutedMode", "UseH46026PriorityQueue" },
+#endif
 	{ "RoutedMode", "UseProvisionalRespToH245Tunneling" },
 	{ "Routing::CatchAll", "CatchAllAlias" },
 	{ "Routing::CatchAll", "CatchAllIP" },
@@ -587,8 +617,19 @@ const char * KnownConfigEntries[][2] = {
 	{ "SyslogAcct", "SyslogLevel" },
 	{ "SyslogAcct", "TimestampFormat" },
 	{ "SyslogAcct", "UpdateEvent" },
+#ifdef HAS_TLS
+	{ "TLS", "CADir" },
+	{ "TLS", "CAFile" },
+	{ "TLS", "Certificates" },
+	{ "TLS", "CheckCertificateIP" },
+	{ "TLS", "EnableTLS" },
+	{ "TLS", "Passphrase" },
+	{ "TLS", "PrivateKey" },
+#endif
 
 	// ignore name partially to check
+	{ "EP::", "AddNumbers" },
+	{ "EP::", "AdditionalDestinationAlias" },
 	{ "EP::", "CalledTypeOfNumber" },
 	{ "EP::", "CallingTypeOfNumber" },
 	{ "EP::", "Capacity" },
@@ -605,6 +646,9 @@ const char * KnownConfigEntries[][2] = {
 	{ "EP::", "Proxy" },
 	{ "EP::", "TranslateReceivedQ931Cause" },
 	{ "EP::", "TranslateSentQ931Cause" },
+#ifdef HAS_TLS
+	{ "EP::", "UseTLS" },
+#endif
 	{ "Neighbor::", "AcceptForwardedLRQ" },
 	{ "Neighbor::", "AcceptPrefixes" },
 	{ "Neighbor::", "AuthUser" },
@@ -615,12 +659,17 @@ const char * KnownConfigEntries[][2] = {
 	{ "Neighbor::", "GatekeeperIdentifier" },
 #ifdef HAS_H46018
 	{ "Neighbor::", "H46018Client" },
+	{ "Neighbor::", "H46018Server" },
 #endif
 	{ "Neighbor::", "Host" },
 	{ "Neighbor::", "Password" },
 	{ "Neighbor::", "SendAuthUser" },
+	{ "Neighbor::", "SendIPs" },
 	{ "Neighbor::", "SendPassword" },
 	{ "Neighbor::", "SendPrefixes" },
+#ifdef HAS_TLS
+	{ "Neighbor", "UseTLS" },
+#endif
 
 	// uncheckable sections
 	{ "CapacityControl", "*" },
@@ -628,7 +677,9 @@ const char * KnownConfigEntries[][2] = {
 	{ "FileIPAuth", "*" },
 	{ "GkStatus::Auth", "*" },
 	{ "H225toQ931", "*" },
+#ifdef H323_H350
 	{ "H350PasswordAuth", "*" },
+#endif
 	{ "ModeSelection", "*" },
 	{ "NATedEndpoints", "*" },
 	{ "PrefixAuth", "*" },
@@ -677,10 +728,17 @@ void ShutdownHandler()
 	// delete singleton objects
 	PTRACE(3, "GK\tDeleting global reference tables");
 
+	// end all calls before deleting handler objects (won't end calls if DisconnectCallsOnShutdown=0)
+	CallTable::Instance()->ClearTable();
+
 	Job::StopAll();
 #ifdef HAS_H46018
 	if (MultiplexedRTPHandler::InstanceExists())
 		delete MultiplexedRTPHandler::Instance();
+#endif
+#ifdef HAS_H46026
+	if (H46026RTPHandler::InstanceExists())
+		delete H46026RTPHandler::Instance();
 #endif
 	if (CapacityControl::InstanceExists())
 		delete CapacityControl::Instance();
@@ -712,6 +770,7 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 	for (PINDEX i = 0; i < sections.GetSize(); ++i) {
 		// check section names
 		PCaselessString sect = sections[i];
+		PString fullSectionName = sections[i];
 		if ((sect.Left(1) == ";") || (sect.Left(1) == "#")) {
 			continue;
 		}
@@ -749,12 +808,12 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 			//PTRACE(0, "Section " << sect << " can't be checked in detail");
 		} else {
 			// check all entries in this section
-			PStringToString entries = cfg->GetAllKeyValues(sect);
+			PStringToString entries = cfg->GetAllKeyValues(fullSectionName);
 			for (PINDEX j = 0; j < entries.GetSize(); j++) {
 				PCaselessString key = entries.GetKeyAt(j);
 				PString value = entries.GetDataAt(j);
 				if (value.IsEmpty()) {
-					PTRACE(2, "WARNING: Empty entry: [" << sect << "] " << key << "=");
+					PTRACE(2, "WARNING: Empty entry: [" << fullSectionName << "] " << key << "=");
 				}
 				unsigned k = 0;
 				bool entry_found = false;
@@ -767,9 +826,9 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 					}
 				}
 				if (!entry_found) {
-					cerr << "WARNING: Config entry [" << sect << "] " << key << "=" << value << " unknown" << endl;
-					PTRACE(0, "WARNING: Config entry [" << sect << "] " << key << "=" << value << " unknown");
-					SNMP_TRAP(7, SNMPError, Configuration, "Config entry [" + sect + "] " + key + " unknown");
+					cerr << "WARNING: Config entry [" << fullSectionName << "] " << key << "=" << value << " unknown" << endl;
+					PTRACE(0, "WARNING: Config entry [" << fullSectionName << "] " << key << "=" << value << " unknown");
+					SNMP_TRAP(7, SNMPError, Configuration, "Config entry [" + fullSectionName + "] " + key + " unknown");
 					warnings++;
 				}
 			}
@@ -793,7 +852,7 @@ void ExitGK()
 	delete Toolkit::Instance();
 
 	Gatekeeper::CloseLogFile();
-	exit(0);
+	_exit(0);	// skip exit handlers: we know GnuGk couldn't start, so avoid crash in useless cleanup
 }
 
 } // end of anonymous namespace
@@ -884,7 +943,7 @@ BOOL WINAPI WinCtrlHandlerProc(DWORD dwCtrlType)
 		eventName = "CTRL_SHUTDOWN_EVENT";
 
 #ifndef hasPTLibTraceOnShutdownBug
-	PTRACE(1, "GK\tGatekeeper shutdown due to "<<eventName);
+	PTRACE(1, "GK\tGatekeeper shutdown due to " << eventName);
 #endif
 
 	PWaitAndSignal shutdown(ShutdownMutex);
@@ -928,6 +987,24 @@ BOOL SetDEP(__in DWORD dwFlags = PROCESS_DEP_ENABLE)
 	return procSet(dwFlags);
 }
 
+void InitializeRTPSending()
+{
+#if (_WIN32_WINNT >= WINDOWS_VISTA)
+	// fetch function pointer for WSASendMsg function
+	// this also servers as runtime check if the feature is available on the machine
+	SOCKET s = socket(AF_INET, SOCK_DGRAM, 0);
+	GUID WSASendMsgGuid = WSAID_WSASENDMSG;
+	DWORD nbytes;
+	int result;
+
+	/* WSASendMsg(): Windows Vista / Windows Server 2008 and later */
+	result = WSAIoctl(s, SIO_GET_EXTENSION_FUNCTION_POINTER,
+						&WSASendMsgGuid, sizeof(WSASendMsgGuid),
+						&g_pfWSASendMsg, sizeof(g_pfWSASendMsg), &nbytes, NULL, NULL);
+	closesocket(s);
+#endif
+}
+
 #else	// _WIN32
 
 #include <pwd.h>
@@ -992,6 +1069,7 @@ Gatekeeper::Gatekeeper(const char * _manuf,
 #ifdef _WIN32
 	// set data execution prevention (ignore if not available)
 	SetDEP(PROCESS_DEP_ENABLE);
+	InitializeRTPSending();	// check for Vista+ method
 #endif
 
 #ifdef COMPILE_AS_SERVICE
@@ -1312,6 +1390,7 @@ void Gatekeeper::Main()
 	vector<PIPSocket::Address> GKHome;
 	PString home(Toolkit::Instance()->GetGKHome(GKHome));
 	if (GKHome.empty()) {
+		PTRACE(0, "Fatal: Cannot find any interface to run GnuGk!");
 		cerr << "Fatal: Cannot find any interface to run GnuGk!\n";
 		ExitGK();
 	}
@@ -1386,7 +1465,7 @@ void Gatekeeper::Main()
 	RasSrv->SetRDSServers();
 
 #ifdef HAS_SNMP
-	if (SelectSNMPImplementation() == "PTLib")
+	if (Toolkit::Instance()->IsSNMPEnabled() && SelectSNMPImplementation() == "PTLib")
 		SNMP_TRAP(1, SNMPInfo, General, "GnuGk started");	// when NOT registering as agent, send started trap here already
 #endif
 
@@ -1402,8 +1481,6 @@ void Gatekeeper::Main()
 	// let's go
 	RasSrv->Run();
 
-	//HouseKeeping();
-
 	// graceful shutdown
 	cerr << "\nShutting down gatekeeper . . . ";
 
@@ -1415,6 +1492,10 @@ void Gatekeeper::Main()
 	SetConsoleCtrlHandler((PHANDLER_ROUTINE)WinCtrlHandlerProc, FALSE);
 	FreeConsole();
 #endif // _WIN32
+
+#ifdef P_OPENBSD
+	_exit(0);	// skip exit handlers, will hang on OpenBSD 5.3
+#endif
 }
 
 namespace {
