@@ -2,7 +2,7 @@
 //
 // gk.h gatekeeper process
 //
-// Copyright (c) 2000-2010, Jan Willamowius
+// Copyright (c) 2000-2013, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -14,7 +14,7 @@
 
 
 #ifndef GK_H
-#define GK_H "@(#) $Id: gk.h,v 1.43 2012/05/23 08:57:32 shorne Exp $"
+#define GK_H "@(#) $Id: gk.h,v 1.45 2013/09/09 16:00:35 willamowius Exp $"
 #include "config.h"
 #include "version.h"
 #ifdef COMPILE_AS_SERVICE
@@ -23,6 +23,10 @@
 #include <ptlib/pprocess.h>
 #endif
 
+#ifdef _WIN32
+#include <mswsock.h>
+#endif
+
 #ifdef COMPILE_AS_SERVICE
 #define GNUGK_NAME	"GNU Gatekeeper"
 #else
@@ -32,6 +36,12 @@
 extern PSemaphore ShutdownMutex;
 extern const char * KnownConfigEntries[][2];
 
+// you must change PTLib 2.10.x configure.ac to set WINVER = 0x0600 to enable
+#define WINDOWS_VISTA	0x0600
+#if defined(_WIN32) && (_WIN32_WINNT >= WINDOWS_VISTA)
+extern LPFN_WSASENDMSG g_pfWSASendMsg;
+#endif
+
 class GkTimer;
 
 #ifdef COMPILE_AS_SERVICE
@@ -71,9 +81,7 @@ class Gatekeeper : public PProcess
 		RotationIntervalMax
 	};
 	
-	static bool SetLogFilename(
-		const PString& filename
-		);
+	static bool SetLogFilename(const PString & filename);
 
 		
 	static bool RotateLogFile();
