@@ -13,7 +13,7 @@
 
 
 #ifndef GK_CONST_H
-#define GK_CONST_H "@(#) $Id: gk_const.h,v 1.30 2013/05/06 08:52:59 willamowius Exp $"
+#define GK_CONST_H "@(#) $Id: gk_const.h,v 1.32 2013/08/02 22:12:08 willamowius Exp $"
 
 #include "config.h"
 
@@ -27,6 +27,7 @@
 
 /* port used by gatekeeper for routed signaling: anything != 1720 so endpoint can be on same IP as GK */
 #define GK_DEF_CALL_SIGNAL_PORT		1721
+#define GK_DEF_TLS_CALL_SIGNAL_PORT	1300
 
 /* well known signal port */
 #define GK_DEF_ENDPOINT_SIGNAL_PORT	1720
@@ -95,6 +96,8 @@ static const char * const H46024B_OID = "0.0.8.460.24.2";
 
 ///////////////////////////////////////////
 // Presence
+
+#define OID3 "1.3.6.1.4.1.17090.0.12"  // Presence v2+
 #define OID3_PDU	1	// PASN_OctetString Presence PDU
 
 
