@@ -5,7 +5,7 @@
  * support for accounting to the gatekeeper.
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
- * Copyright (c) 2005-2012, Jan Willamowius
+ * Copyright (c) 2005-2013, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -207,6 +207,11 @@ void GkAcctLogger::SetupAcctParams(
 	params["codec"] = call->GetCodec();
 	params["bandwidth"] = call->GetBandwidth();
 	params["client-auth-id"] = call->GetClientAuthId();
+	PString vendor, version;
+	call->GetCallingVendor(vendor, version);
+	params["caller-vendor"] = vendor + " " + version;
+	call->GetCalledVendor(vendor, version);
+	params["callee-vendor"] = vendor + " " + version;
 }
 
 void GkAcctLogger::SetupAcctEndpointParams(
@@ -964,7 +969,7 @@ bool GkAcctLoggerList::LogAcctEvent(
 		PTrace::End(strm);
 #ifdef HAS_SNMP
 		if (!finalResult)
-			SNMP_TRAP(7, SNMPError, Accounting, "Failed to log event " + evt);
+			SNMP_TRAP(7, SNMPError, Accounting, "Failed to log event " + PString(PString::Unsigned, evt));
 #endif
 	}
 	return finalResult;
@@ -1034,7 +1039,7 @@ bool GkAcctLoggerList::LogAcctEvent(
 		PTrace::End(strm);
 #ifdef  HAS_SNMP
 		if (!finalResult)
-			SNMP_TRAP(7, SNMPError, Accounting, "Failed to log event " + evt);
+			SNMP_TRAP(7, SNMPError, Accounting, "Failed to log event " + PString(PString::Unsigned, evt));
 #endif
 	}
 	return finalResult;
