@@ -14,7 +14,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKAUTH_H
-#define GKAUTH_H "@(#) $Id: gkauth.h,v 1.62 2013/02/23 12:01:55 shorne Exp $"
+#define GKAUTH_H "@(#) $Id: gkauth.h,v 1.67 2013/08/04 04:21:10 willamowius Exp $"
 
 #include <map>
 #include <list>
@@ -143,7 +143,9 @@ struct SetupAuthData
 		/// call associated with the message (if any)
 		const callptr& call,
 		/// is the Setup message from a registered endpoint
-		bool fromRegistered
+		bool fromRegistered,
+		/// did the Setup come in over TLS
+		bool overTLS = false
 		);
 	virtual ~SetupAuthData();
 		
@@ -180,6 +182,8 @@ struct SetupAuthData
 	PBYTEArray m_radiusClass;
 	/// ID provided by client, to be passed out with accounting events
 	PUInt64 m_clientAuthId;
+	/// True if Setup came in over TLS
+	bool m_overTLS;
 		
 private:
 	SetupAuthData();
@@ -479,7 +483,7 @@ class CacheManager
 public:
 	CacheManager(
 		long timeout = -1 /// cache timeout - expiry period (seconds)
-		) : m_ttl(timeout) {}
+		) : m_ttl(timeout) { }
 
 	/** Get a value associated with the key.
 	
@@ -488,14 +492,14 @@ public:
 	    false if the key-value pair is not cached or the cache expired
 	*/
 	bool Retrieve(
-		const PString& key, /// the key to look for
-		PString& value /// filled with the value on return
+		const PString & key, /// the key to look for
+		PString & value /// filled with the value on return
 		) const;
 		
 	/// Store a key-value association in the cache
 	void Save(
-		const PString& key, /// a key to be stored
-		const PString& value /// a value to be associated with the key
+		const PString & key, /// a key to be stored
+		const PString & value /// a value to be associated with the key
 		);
 
 	void SetTimeout(
@@ -503,8 +507,8 @@ public:
 		) { m_ttl = newTimeout; }
 
 private:
-	CacheManager(const CacheManager&);
-	CacheManager & operator=(const CacheManager&);
+	CacheManager(const CacheManager &);
+	CacheManager & operator=(const CacheManager &);
 	
 private:
 	/// cache timeout (seconds), 0 = do not cache, -1 = never expires
@@ -638,6 +642,8 @@ protected:
 	bool ResolveUserName(
 		/// an array of tokens to be checked
 		const H225_CryptoH323Token & crytotoken,
+		/// the aliases of the request (only used for cryptoEPPwdEncr)
+		const H225_ArrayOf_AliasAddress * aliases,
 		/// UserName detected.
 		PString & username
 		);
@@ -661,8 +667,7 @@ protected:
 	{
 		const RAS & req = request;
 		bool finalResult = false;
-		
-#ifdef OpenH323Factory
+
 		if (m_h235Authenticators == NULL) {
 			PTRACE(4, "GKAUTH\tNo Loaded Authenticators");
 			return GetDefaultStatus();
@@ -674,8 +679,8 @@ protected:
 
 			H235Authenticator::ValidationResult result;
 			for (PINDEX t = 0; t < req.m_tokens.GetSize(); t++) {
-				PString username = PString::Empty();
-				PString password = PString::Empty();
+				PString username;
+				PString password;
 				if (!ResolveUserName(req.m_tokens[t], username)) {
 		            PTRACE(4, "GKAUTH\t" << GetName() << " No username resolved from tokens.");
 					continue;	// skip to next token
@@ -699,9 +704,9 @@ protected:
 				}
 			}
 			for (PINDEX t = 0; t < req.m_cryptoTokens.GetSize(); t++) {
-				PString username = PString::Empty();
-				PString password = PString::Empty();
-				if (!ResolveUserName(req.m_cryptoTokens[t], username)) {
+				PString username;
+				PString password;
+				if (!ResolveUserName(req.m_cryptoTokens[t], aliases, username)) {
 		            PTRACE(4, "GKAUTH\t" << GetName() << " No username resolved from tokens.");
 					continue;	// skip to next token
 				}
@@ -715,6 +720,7 @@ protected:
 				}
 				authenticator->SetRemoteId(username);
 				authenticator->SetPassword(password);
+				//authenticator->SetChallenge(challengeToken);	// TODO: set challenge token from GCF
 				result = authenticator->ValidateCryptoToken(req.m_cryptoTokens[t], request->m_rasPDU);
 				if (result == H235Authenticator::e_OK) {
 					PTRACE(4, "GKAUTH\tAuthenticator " << authenticator->GetName() << " succeeded");
@@ -724,19 +730,7 @@ protected:
 				}
 			}
 		}
-#else
-		int result;
-		if (req.HasOptionalField(RAS::e_cryptoTokens)) {
-			if ((result = CheckCryptoTokens(req.m_cryptoTokens, aliases, request->m_rasPDU)) == e_fail)
-				return e_fail;
-			finalResult = (result == e_ok);
-		}
-		if (req.HasOptionalField(RAS::e_tokens)) {
-			if ((result = CheckTokens(req.m_tokens, aliases)) == e_fail)
-				return e_fail;
-			finalResult = finalResult || (result == e_ok);
-		}
-#endif
+
 		return finalResult ? e_ok : GetDefaultStatus();
 	}
 
@@ -1022,8 +1016,8 @@ public:
 	}
 
 private:
-	GkAuthenticatorList(const GkAuthenticatorList&);
-	GkAuthenticatorList& operator=(const GkAuthenticatorList&);
+	GkAuthenticatorList(const GkAuthenticatorList& );
+	GkAuthenticatorList & operator=(const GkAuthenticatorList &);
 
 private:
 	/// a list of all configured authenticators
@@ -1032,12 +1026,7 @@ private:
 	PReadWriteMutex m_reloadMutex;
 	/// the most common authentication capabilities 
 	/// shared by all authenticators on the list
-#ifdef OpenH323Factory
     H235Authenticators m_h235authenticators;
-#else
-	H225_ArrayOf_AuthenticationMechanism* m_mechanisms;
-	H225_ArrayOf_PASN_ObjectId* m_algorithmOIDs;
-#endif
 };
 
 /** A factory template for authenticator objects. When you create
@@ -1055,7 +1044,7 @@ private:
 */
 template<class Auth>
 struct GkAuthCreator : public Factory<GkAuthenticator>::Creator0 {
-	GkAuthCreator(const char *n) : Factory<GkAuthenticator>::Creator0(n) {}
+	GkAuthCreator(const char *n) : Factory<GkAuthenticator>::Creator0(n) { }
 	virtual GkAuthenticator *operator()() const { return new Auth(m_id); }
 };
 
