@@ -17,6 +17,10 @@
 #include "config.h"
 #include "h323util.h"
 
+#ifdef HAS_H460
+#include <h460/h4601.h>
+#endif
+
 
 #if defined(_WIN32) && defined(hasIPV6)
 // Windows doesn't have inet_ntop, so we fake it
@@ -209,7 +213,6 @@ PString AsString(const H225_AliasAddress & terminalAlias, bool includeAliasType)
 				aliasString += ":" + terminalAlias.GetTagName();
 			}
 			return aliasString;
-			break;
 	}
 
 	return includeAliasType ? "none:UnknownType" : "none";
@@ -218,7 +221,7 @@ PString AsString(const H225_AliasAddress & terminalAlias, bool includeAliasType)
 
 PString AsString(const H225_ArrayOf_AliasAddress & terminalAlias, bool includeAliasTypeName)
 {
-	PString aliasListString = PString::Empty();
+	PString aliasListString;
 
 	for(PINDEX cnt = 0; cnt < terminalAlias.GetSize(); cnt++ )
 	{
@@ -351,6 +354,20 @@ H225_TransportAddress SocketToH225TransportAddr(const PIPSocket::Address & Addr,
 	return Result;
 }
 
+// convert a H.245 unicast address into an H.323 transport address
+H323TransportAddress H245UnicastToH323TransportAddr(const H245_UnicastAddress & h245unicast)
+{
+	if (h245unicast.GetTag() == H245_UnicastAddress::e_iPAddress) {
+		const H245_UnicastAddress_iPAddress & ipv4 = h245unicast;
+		return H323TransportAddress(PIPSocket::Address(ipv4.m_network.GetSize(), ipv4.m_network.GetValue()), ipv4.m_tsapIdentifier);
+	} else if (h245unicast.GetTag() == H245_UnicastAddress::e_iP6Address) {
+		const H245_UnicastAddress_iP6Address & ipv6 = h245unicast;
+		return H323TransportAddress(PIPSocket::Address(ipv6.m_network.GetSize(), ipv6.m_network.GetValue()), ipv6.m_tsapIdentifier);
+	}
+	PTRACE(1, "Unsupported H245_UnicastAddress: " << h245unicast.GetTagName());
+	return H323TransportAddress();
+}
+
 bool GetTransportAddress(const PString & addr, WORD def_port, PIPSocket::Address & ip, WORD & port)
 {
 	PStringArray adr_parts = SplitIPAndPort(addr.Trim(), def_port);
@@ -708,9 +725,11 @@ int MatchPrefix(
 }
 
 PString RewriteString(
-	const PString & s, /// original string to rewrite
-	const char* prefix, /// prefix string that matched
-	const char* value /// new string that replaces the prefix string
+	const PString & s,	/// original string to rewrite
+	const char* prefix,	/// prefix string that matched
+	const char* value,	/// new string that replaces the prefix string
+	PString & postdialdigits,
+	bool postdialmatch
 	)
 {
 	if (prefix == NULL || value == NULL)
@@ -718,24 +737,33 @@ PString RewriteString(
 	
 	PString result = value + s.Mid(strlen(prefix));
 
+	if (postdialmatch)
+		postdialdigits.MakeEmpty();	// clear postdial digits before assigning new digits
+
 	const char *lastSrcDot = prefix;
 	const char *lastDstDot = strchr(value, '.');
 	while (lastDstDot != NULL) {
 		lastSrcDot = strchr(lastSrcDot, '.');
 		if (lastSrcDot == NULL) {
-			PTRACE(1, "GK\tInvalid rewrite rule (dots do not match) - "
-				<< prefix << " = " << value
-				);
+			PTRACE(1, "GK\tInvalid rewrite rule (dots do not match) - " << prefix << " = " << value);
 			break;
 		}
 		int dotDstOffset = (long)lastDstDot - (long)value;
 		int dotSrcOffset = (long)lastSrcDot - (long)prefix;
-		while (*lastDstDot++ == '.' && *lastSrcDot++ == '.')
-			result[dotDstOffset++] = s[dotSrcOffset++];
-						
+		while (*lastDstDot++ == '.' && *lastSrcDot++ == '.') {
+			if (postdialmatch) {
+				postdialdigits += s[dotSrcOffset++];
+			} else {
+				result[dotDstOffset++] = s[dotSrcOffset++];
+			}
+		}
+				
 		lastDstDot = strchr(lastDstDot, '.');
 	}
-	
+
+	if (postdialmatch)
+		result.Replace(".", "", true);
+
 	return result;
 }
 
@@ -808,4 +836,43 @@ PString ReplaceParameters(const PString & queryStr, const std::map<PString, PStr
 	return finalQuery;
 }
 
+bool FindH460Descriptor(unsigned feat, H225_ArrayOf_FeatureDescriptor & features, unsigned & location)
+{
+    for (PINDEX i = 0; i < features.GetSize(); i++) {
+        H225_GenericIdentifier & id = features[i].m_id;
+        if (id.GetTag() == H225_GenericIdentifier::e_standard) {
+            PASN_Integer & asnInt = id;
+            if (asnInt.GetValue() == feat) {
+                location = i;
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
+void RemoveH460Descriptor(unsigned feat, H225_ArrayOf_FeatureDescriptor & features)
+{
+    for(PINDEX i = 0; i < features.GetSize(); i++) {
+        H225_GenericIdentifier & id = features[i].m_id;
+        if (id.GetTag() == H225_GenericIdentifier::e_standard) {
+            PASN_Integer & asnInt = id;
+            if (asnInt.GetValue() == feat) {
+                for(PINDEX j=i+1; j < features.GetSize(); j++)
+                    features[j-1] = features[j];
+                features.SetSize(features.GetSize() - 1);
+                return;
+            }   
+        }
+    }
+}
+
+#ifdef HAS_H460
+void AddH460Feature(H225_ArrayOf_FeatureDescriptor & desc, const H460_Feature & newFeat)
+{
+    PINDEX lastpos = desc.GetSize();
+    desc.SetSize(lastpos+1);
+    desc[lastpos] = newFeat;
+}
+#endif
 
