@@ -2,7 +2,7 @@
 //
 // H.323 utility functions
 //
-// Copyright (c) 2000-2012, Jan Willamowius
+// Copyright (c) 2000-2013, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,7 +13,7 @@
 
 
 #ifndef H323UTIL_H
-#define H323UTIL_H "@(#) $Id: h323util.h,v 1.50 2013/04/17 05:29:58 shorne Exp $"
+#define H323UTIL_H "@(#) $Id: h323util.h,v 1.54 2013/09/12 10:51:58 willamowius Exp $"
 
 #include <ptlib.h>
 #include <ptlib/sockets.h>
@@ -33,6 +33,7 @@ class H225_ArrayOf_AliasAddress;
 class H245_TransportAddress;
 class H245_UnicastAddress;
 class PASN_OctetString;
+class H460_Feature;
 
 PString AsString(const PIPSocket::Address &);
 
@@ -73,6 +74,9 @@ H245_TransportAddress IPToH245TransportAddr(const PIPSocket::Address & ip, WORD
 // convert a socket IP address into an H225 transport address
 H225_TransportAddress SocketToH225TransportAddr(const PIPSocket::Address & Addr, WORD Port);
 
+// convert a H.245 unicast address into an H.323 transport address
+H323TransportAddress H245UnicastToH323TransportAddr(const H245_UnicastAddress & h245unicast);
+
 bool GetTransportAddress(const PString & addr, WORD def_port, PIPSocket::Address & ip, WORD & port);
 
 bool GetTransportAddress(const PString & addr, WORD def_port, H225_TransportAddress & Result);
@@ -216,11 +220,13 @@ int MatchPrefix(
     @return	Rewritten string.
 */
 PString RewriteString(
-	const PString& s, /// original string to rewrite
+	const PString& s,	/// original string to rewrite
 	const char *prefix, /// prefix string that matched
-	const char *value /// new string that replaces the prefix string
+	const char *value,	/// new string that replaces the prefix string
+	PString & postdialdigts,
+	bool postdialmatch = false
 	);
-	
+
 /** Rewrite Wildcard match 
 	Lightweight Regex match rewrite.
 	Only basic number matching supported at the moment
@@ -245,6 +251,12 @@ PString ReplaceParameters(
 	const std::map<PString, PString> & queryParams	/// parameter values
 	);
 
+bool FindH460Descriptor(unsigned feat, H225_ArrayOf_FeatureDescriptor & features, unsigned & location);
+
+void RemoveH460Descriptor(unsigned feat, H225_ArrayOf_FeatureDescriptor & features);
+
+#ifdef HAS_H460
+void AddH460Feature(H225_ArrayOf_FeatureDescriptor & desc, const H460_Feature & newFeat);
+#endif
 
-	
 #endif // H323UTIL_H
