@@ -13,7 +13,7 @@
 
 
 #ifndef PWLIB_COMPAT_H
-#define PWLIB_COMPAT_H "@(#) $Id: pwlib_compat.h,v 1.80 2013/04/29 18:04:47 shorne Exp $"
+#define PWLIB_COMPAT_H "@(#) $Id: pwlib_compat.h,v 1.85 2013/08/04 00:30:50 willamowius Exp $"
 
 #include "ptbuildopts.h"
 #include "openh323buildopts.h"
@@ -145,6 +145,10 @@
 	#endif
 #endif
 
+#if defined(P_SSL)
+	#define HAS_TLS 1
+#endif
+
 #if defined(hasPTRACE2)
    #define PTRACEX(level, args)  PTRACE2(level,NULL,args)
 #else
@@ -181,6 +185,7 @@
 
 #ifdef H323_H460
 	#define HAS_H460 1    // H460 support
+	#define HAS_H460VEN	1
 #else
 	#undef HAS_H46017
 	#undef HAS_H46018
@@ -206,16 +211,19 @@
 
 #ifdef H323_H460P
 	#define HAS_H460P	1  // Presence
-#if H323PLUS_VER > 1251
-	#define H460P_VER 3
+#ifdef H323_H460P_VER
+  #if H323_H460P_VER == 3
+	#define HAS_H460P_VER_3	 1
+  #else
+	#define HAS_H460P_VER_2	 1
+  #endif
 #else
-	#define H460P_VER 2
+	#define HAS_H460P_VER_1  1
 #endif
-	#define OID3 "1.3.6.1.4.1.17090.0.12"  // Presence v2+
-//#else
-//	#define H460P_VER 1
-//	#define OID3 "1.3.6.1.4.1.17090.0.3"  // Presence
-//#endif
+#endif
+
+#ifdef H323_H460PRE
+	#define HAS_H460PRE 1
 #endif
 
 #if OPENH323_MAJOR == 1
@@ -234,15 +242,11 @@
 #if OPENH323_MAJOR == 1
 	#if OPENH323_MINOR >= 19
 		#if OPENH323_MINOR == 19
-			#if OPENH323_BUILD > 0
-				#define OpenH323Factory 1	// OpenH323 Factory Loader Auth
-			#endif
 			#if OPENH323_BUILD > 4
 				#define h323pluslib 1		// Indicate H323plus Library
 				#define h323v6 1			// Version 6 features
 			#endif
 		#else // h323plus v1.20
-			#define OpenH323Factory 1		// OpenH323 Factory Loader Auth 
 			#define h323pluslib 1			// Indicate H323plus Library
 			#define h323v6 1				// Version 6 features  
 		#endif
