@@ -5,7 +5,7 @@
  * Please see docs/radauth.txt for more details.
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
- * Copyright (c) 2005-2011, Jan Willamowius
+ * Copyright (c) 2005-2012, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -101,9 +101,9 @@ RadAuthBase::~RadAuthBase()
 
 int RadAuthBase::Check(
 	/// RRQ RAS message to be authenticated
-	RasPDU<H225_RegistrationRequest>& rrqPdu, 
+	RasPDU<H225_RegistrationRequest> & rrqPdu, 
 	/// authorization data (reject reason, ...)
-	RRQAuthData& authData
+	RRQAuthData & authData
 	)
 {
 	H225_RegistrationRequest& rrq = (H225_RegistrationRequest&)rrqPdu;
@@ -147,8 +147,8 @@ int RadAuthBase::Check(
 	}
 	if (!ipFound) {
 		PTRACE(2, "RADAUTH\t" << GetName() << " RRQ auth failed: "
-			"could not determine Framed-IP-Address"
-			);
+			"could not determine Framed-IP-Address");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " RRQ check failed");
 		authData.m_rejectReason = H225_RegistrationRejectReason::e_invalidCallSignalAddress;
 		delete pdu;
 		return e_fail;
@@ -170,10 +170,6 @@ int RadAuthBase::Check(
 			);
 	}
 	
-	if (!OnSendPDU(*pdu, rrqPdu, authData)) {
-		delete pdu;
-		return e_fail;
-	}
 	// send request and wait for response
 	RadiusPDU* response = NULL;
 	bool result = m_radiusClient->MakeRequest(*pdu, response) && response;
@@ -182,8 +178,8 @@ int RadAuthBase::Check(
 			
 	if (!result) {
 		PTRACE(2, "RADAUTH\t" << GetName() << " RRQ auth failed: "
-			" could not receive or decode response from RADIUS"
-			);
+			" could not receive or decode response from RADIUS");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " RRQ check failed");
 		delete response;
 		response = NULL;
 		authData.m_rejectReason = H225_RegistrationRejectReason::e_undefinedReason;
@@ -206,15 +202,14 @@ int RadAuthBase::Check(
 				&& strspn((const char*)value, "0123456789") == (size_t)value.GetLength()) {
 				const unsigned retcode = value.AsUnsigned();
 				if (retcode != 0) {
-					PTRACE(3, "RADAUTH\t" << GetName() << " RRQ check failed: "
-						"return code " << retcode
-						);
+					PTRACE(3, "RADAUTH\t" << GetName() << " RRQ check failed: return code " << retcode);
+					SNMP_TRAP(8, SNMPError, Authentication, GetName() + " RRQ check failed");
 					result = false;
 				}
 			} else {
 				PTRACE(2, "RADAUTH\t" << GetName() << " RRQ check failed: "
-					"invalid h323-return-code attribute '" << value << '\''
-					);
+					"invalid h323-return-code attribute '" << value << '\'');
+				SNMP_TRAP(8, SNMPError, Authentication, GetName() + " RRQ check failed");
 				result = false;
 			}
 		}
@@ -320,9 +315,7 @@ int RadAuthBase::Check(
 		}
 	}
 
-	if (result)
-		result = OnReceivedPDU(*response, rrqPdu, authData);
-	else
+	if (!result)
 		authData.m_rejectReason = H225_RegistrationRejectReason::e_securityDenial;
 				
 	delete response;
@@ -365,8 +358,8 @@ int RadAuthBase::Check(
 		delete pdu;
 		PTRACE(3, "RADAUTH\t" << GetName() << " ARQ auth failed: "
 			"requesting endpoint " << arq.m_endpointIdentifier 
-			<< " not registered"
-			);
+			<< " not registered");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " ARQ check failed");
 		authData.m_rejectReason = arq.m_answerCall
 			? H225_AdmissionRejectReason::e_calledPartyNotRegistered
 			: H225_AdmissionRejectReason::e_callerNotRegistered;
@@ -422,8 +415,8 @@ int RadAuthBase::Check(
 	}
 	if (!ipFound) {
 		PTRACE(2, "RADAUTH\t" << GetName() << " ARQ auth failed: "
-			"could not setup Framed-IP-Address"
-			);
+			"could not setup Framed-IP-Address");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " ARQ check failed");
 		authData.m_rejectReason = H225_AdmissionRejectReason::e_securityDenial;
 		delete pdu;
 		return e_fail;
@@ -443,8 +436,8 @@ int RadAuthBase::Check(
 	if (stationId.IsEmpty()) {
 		delete pdu;
 		PTRACE(2, "RADAUTH\t" << GetName() << " ARQ auth failed: "
-			"no suitable alias for Calling-Station-Id has been found"
-			);
+			"no suitable alias for Calling-Station-Id has been found");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " ARQ check failed");
 		authData.m_rejectReason = H225_AdmissionRejectReason::e_securityDenial;
 		return e_fail;
 	} else
@@ -463,10 +456,6 @@ int RadAuthBase::Check(
 		pdu->AppendAttr(m_attrH323GwId);
 	}
 				
-	if (!OnSendPDU(*pdu, arqPdu, authData)) {
-		delete pdu;
-		return e_fail;
-	}
 	// send the request and wait for a response
 	RadiusPDU* response = NULL;
 	bool result = m_radiusClient->MakeRequest(*pdu, response) && response;
@@ -475,8 +464,8 @@ int RadAuthBase::Check(
 
 	if (!result) {
 		PTRACE(2, "RADAUTH\t" << GetName() << " ARQ auth failed: "
-			" could not receive or decode response from RADIUS"
-			);
+			" could not receive or decode response from RADIUS");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " ARQ check failed");
 		delete response;
 		response = NULL;
 		authData.m_rejectReason = H225_AdmissionRejectReason::e_undefinedReason;
@@ -511,14 +500,14 @@ int RadAuthBase::Check(
 				const unsigned retcode = value.AsUnsigned();
 				if (retcode != 0) {
 					PTRACE(3, "RADAUTH\t" << GetName() << " ARQ check failed: "
-						"return code " << retcode
-						);
+						"return code " << retcode);
+					SNMP_TRAP(8, SNMPError, Authentication, GetName() + " ARQ check failed");
 					result = false;
 				}
 			} else {
 				PTRACE(2, "RADAUTH\t" << GetName() << " ARQ check failed: "
-					"invalid h323-return-code attribute '" << value << '\''
-					);
+					"invalid h323-return-code attribute '" << value << '\'');
+				SNMP_TRAP(8, SNMPError, Authentication, GetName() + " ARQ check failed");
 				result = false;
 			}
 		}
@@ -558,8 +547,8 @@ int RadAuthBase::Check(
 					result = false;
 			} else {
 				PTRACE(2, "RADAUTH\t" << GetName() << " ARQ check failed: "
-					"invalid h323-credit-time attribute '" << value << '\''
-					);
+					"invalid h323-credit-time attribute '" << value << '\'');
+				SNMP_TRAP(8, SNMPError, Authentication, GetName() + " ARQ check failed");
 				result = false;
 			}
 		}
@@ -723,9 +712,7 @@ int RadAuthBase::Check(
 		}
 	}
 
-	if (result)
-		result = OnReceivedPDU(*response, arqPdu, authData);
-	else
+	if (!result)
 		authData.m_rejectReason = H225_AdmissionRejectReason::e_securityDenial;
 					
 	delete response;
@@ -798,8 +785,8 @@ int RadAuthBase::Check(
 	}
 	if (!ipFound) {
 		PTRACE(2, "RADAUTH\t" << GetName() << " Setup auth failed: "
-			"could not setup Framed-IP-Address"
-			);
+			"could not setup Framed-IP-Address");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " Setup check failed");
 		delete pdu;
 		authData.m_rejectCause = Q931::CallRejected;
 		return e_fail;
@@ -819,8 +806,8 @@ int RadAuthBase::Check(
 	if (stationId.IsEmpty()) {
 		delete pdu;
 		PTRACE(2, "RADAUTH\t" << GetName() << " Setup check failed: "
-			"no called station id found"
-			);
+			"no called station id found");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " Setup check failed");
 		authData.m_rejectReason = H225_ReleaseCompleteReason::e_badFormatAddress;
 		return e_fail;
 	} else
@@ -835,10 +822,6 @@ int RadAuthBase::Check(
 		pdu->AppendAttr(m_attrH323GwId);
 	}
 				
-	if (!OnSendPDU(*pdu, setup, authData)) {
-		delete pdu;
-		return e_fail;
-	}
 	// send the request and wait for a response
 	RadiusPDU* response = NULL;
 	bool result = m_radiusClient->MakeRequest(*pdu, response) && response;
@@ -847,8 +830,8 @@ int RadAuthBase::Check(
 			
 	if (!result) {
 		PTRACE(2, "RADAUTH\t" << GetName() << " Setup auth failed: "
-			" could not receive or decode response from RADIUS"
-			);
+			" could not receive or decode response from RADIUS");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " Setup check failed");
 		delete response;
 		response = NULL;
 		authData.m_rejectCause = Q931::TemporaryFailure;
@@ -883,14 +866,14 @@ int RadAuthBase::Check(
 				const unsigned retcode = value.AsUnsigned();
 				if (retcode != 0) {
 					PTRACE(5, "RADAUTH\t" << GetName() << " Setup check failed: "
-						"return code " << retcode
-						);
+						"return code " << retcode);
+					SNMP_TRAP(8, SNMPError, Authentication, GetName() + " Setup check failed");
 					result = false;
 				}
 			} else {
 				PTRACE(2, "RADAUTH\t" << GetName() << " Setup check failed: "
-					"invalid h323-return-code attribute '" << value << '\''
-					);
+					"invalid h323-return-code attribute '" << value << '\'');
+				SNMP_TRAP(8, SNMPError, Authentication, GetName() + " Setup check failed");
 				result = false;
 			}
 		}
@@ -930,8 +913,8 @@ int RadAuthBase::Check(
 					result = false;
 			} else {
 				PTRACE(2, "RADAUTH\t" << GetName() << " Setup check failed: "
-					"invalid h323-credit-time attribute '" << value << '\''
-					);
+					"invalid h323-credit-time attribute '" << value << '\'');
+				SNMP_TRAP(8, SNMPError, Authentication, GetName() + " Setup check failed");
 				result = false;
 			}
 		}
@@ -1043,9 +1026,7 @@ int RadAuthBase::Check(
 		}
 	}
 
-	if (result)
-		result = OnReceivedPDU(*response, setup, authData);
-	else
+	if (!result)
 		authData.m_rejectCause = Q931::CallRejected;
 					
 	delete response;
@@ -1053,60 +1034,6 @@ int RadAuthBase::Check(
 	return result ? e_ok : e_fail;
 }		
 
-bool RadAuthBase::OnSendPDU(
-	RadiusPDU& /*pdu*/,
-	RasPDU<H225_RegistrationRequest>& /*rrqPdu*/,
-	RRQAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
-bool RadAuthBase::OnSendPDU(
-	RadiusPDU& /*pdu*/,
-	RasPDU<H225_AdmissionRequest>& /*arqPdu*/,
-	ARQAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
-bool RadAuthBase::OnSendPDU(
-	RadiusPDU& /*pdu*/,
-	SetupMsg &/*setup*/,
-	SetupAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
-bool RadAuthBase::OnReceivedPDU(
-	RadiusPDU& /*pdu*/,
-	RasPDU<H225_RegistrationRequest>& /*rrqPdu*/,
-	RRQAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
-bool RadAuthBase::OnReceivedPDU(
-	RadiusPDU& /*pdu*/,
-	RasPDU<H225_AdmissionRequest>& /*arqPdu*/,
-	ARQAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
-bool RadAuthBase::OnReceivedPDU(
-	RadiusPDU& /*pdu*/,
-	SetupMsg &/*setup*/,
-	SetupAuthData& /*authData*/
-	)
-{
-	return true;
-}
-
 int RadAuthBase::AppendUsernameAndPassword(
 	RadiusPDU& /*pdu*/,
 	RasPDU<H225_RegistrationRequest>& /*rrqPdu*/,
@@ -1148,7 +1075,7 @@ RadAuth::RadAuth(
 	// by this authenticator - this will make sure
 	// GCF H.235 alogirthm selection will not skip
 	// information required by this authenticator
-	H235AuthCAT* authenticator = new H235AuthCAT;
+	H235AuthCAT* authenticator = new H235AuthCAT();
 	authenticator->SetLocalId("dummy");
 	authenticator->SetRemoteId("dummy");
 	authenticator->SetPassword("dummy");
@@ -1181,8 +1108,8 @@ int RadAuth::CheckTokens(
 			&& token.HasOptionalField(H235_ClearToken::e_challenge))) 
 		{	
 			PTRACE(3, "RADAUTH\t" << GetName() << " auth failed: "
-				"CAT without all required fields"
-				);
+				"CAT without all required fields");
+			SNMP_TRAP(8, SNMPError, Authentication, GetName() + " failed");
 			return e_fail;
 		}
 				
@@ -1190,8 +1117,8 @@ int RadAuth::CheckTokens(
 		const PString id = token.m_generalID;
 		if (aliases && FindAlias(*aliases, id) == P_MAX_INDEX) {
 			PTRACE(3, "RADAUTH\t" << GetName() << " auth failed: "
-				"CAT m_generalID is not a valid alias"
-				);
+				"CAT m_generalID is not a valid alias");
+			SNMP_TRAP(8, SNMPError, Authentication, GetName() + " failed");
 			return e_fail;
 		}
 					
@@ -1199,16 +1126,16 @@ int RadAuth::CheckTokens(
 		const int randomInt = token.m_random;
 		if (randomInt < -127 || randomInt > 255) {
 			PTRACE(3, "RADAUTH\t" << GetName() << " auth failed: "
-				"CAT m_random out of range"
-				);
+				"CAT m_random out of range");
+			SNMP_TRAP(8, SNMPError, Authentication, GetName() + " failed");
 			return e_fail;
 		}
 					
 		// CAT challenge has to be 16 bytes
 		if (token.m_challenge.GetValue().GetSize() < 16) {
 			PTRACE(3, "RADAUTH\t" << GetName() << " auth failed: "
-				"m_challenge less than 16 bytes"
-				);
+				"m_challenge less than 16 bytes");
+			SNMP_TRAP(8, SNMPError, Authentication, GetName() + " failed");
 			return e_fail;
 		}
 					
@@ -1218,7 +1145,7 @@ int RadAuth::CheckTokens(
 			*username = (const char*)id;
 				
 		// build CHAP-Password
-		char password[17] = { (BYTE)randomInt };
+		unsigned char password[17] = { (BYTE)randomInt };
 		memcpy(password + 1, (const BYTE*)(token.m_challenge), 16);
 				
 		pdu.AppendAttr(RadiusAttr::ChapPassword, password, sizeof(password));
@@ -1227,6 +1154,7 @@ int RadAuth::CheckTokens(
 		return e_ok;
 	}
 	PTRACE(3, "RADAUTH\t" << GetName() << " auth failed: no CAT token found");
+	SNMP_TRAP(8, SNMPError, Authentication, GetName() + " failed: no token");
 	return GetDefaultStatus();
 }
 
@@ -1242,8 +1170,8 @@ int RadAuth::AppendUsernameAndPassword(
 	// RRQ has to carry at least one terminalAlias
 	if (!rrq.HasOptionalField(H225_RegistrationRequest::e_terminalAlias)) {
 		PTRACE(3, "RADAUTH\t" << GetName() << " RRQ auth failed: "
-			"no m_terminalAlias field"
-			);
+			"no m_terminalAlias field");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + "RRQ check failed");
 		authData.m_rejectReason = H225_RegistrationRejectReason::e_securityDenial;
 		return GetDefaultStatus();
 	}
@@ -1251,8 +1179,8 @@ int RadAuth::AppendUsernameAndPassword(
 	// check for ClearTokens (CAT uses ClearTokens)
 	if (!rrq.HasOptionalField(H225_RegistrationRequest::e_tokens)) {
 		PTRACE(3, "RADAUTH\t" << GetName() << " RRQ auth failed: "
-			"tokens not found"
-			);
+			"tokens not found");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + "RRQ check failed");
 		authData.m_rejectReason = H225_RegistrationRejectReason::e_securityDenial;
 		return GetDefaultStatus();
 	}
@@ -1264,10 +1192,10 @@ int RadAuth::AppendUsernameAndPassword(
 }
 
 int RadAuth::AppendUsernameAndPassword(
-	RadiusPDU& pdu,
-	RasPDU<H225_AdmissionRequest>& arqPdu,
-	ARQAuthData& authData,
-	PString* username
+	RadiusPDU & pdu,
+	RasPDU<H225_AdmissionRequest> & arqPdu,
+	ARQAuthData & authData,
+	PString * username
 	) const
 {
 	H225_AdmissionRequest& arq = (H225_AdmissionRequest&)arqPdu;
@@ -1275,8 +1203,8 @@ int RadAuth::AppendUsernameAndPassword(
 	// check for ClearTokens
 	if (!arq.HasOptionalField(H225_AdmissionRequest::e_tokens)) {
 		PTRACE(3, "RADAUTH\t" << GetName() << " ARQ auth failed: "
-			"tokens not found"
-			);
+			"tokens not found");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " ARQ check failed");
 		authData.m_rejectReason = H225_AdmissionRejectReason::e_securityDenial;
 		return GetDefaultStatus();
 	}
@@ -1288,17 +1216,18 @@ int RadAuth::AppendUsernameAndPassword(
 }
 
 int RadAuth::AppendUsernameAndPassword(
-	RadiusPDU& pdu,
-	SetupMsg &setup,
-	endptr& /*callingEP*/,
-	SetupAuthData& authData,
-	PString* username
+	RadiusPDU & pdu,
+	SetupMsg & setup,
+	endptr & /*callingEP*/,
+	SetupAuthData & authData,
+	PString * username
 	) const
 {
 	H225_Setup_UUIE &setupBody = setup.GetUUIEBody();
 	// check for ClearTokens (CAT uses ClearTokens)
 	if (!setupBody.HasOptionalField(H225_Setup_UUIE::e_tokens)) {
 		PTRACE(3, "RADAUTH\t" << GetName() << " Setup auth failed: no tokens");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " Setup check failed");
 		authData.m_rejectReason = H225_ReleaseCompleteReason::e_securityDenied;
 		return GetDefaultStatus();
 	}
@@ -1310,17 +1239,16 @@ int RadAuth::AppendUsernameAndPassword(
 }
 
 RadAliasAuth::RadAliasAuth( 
-	const char* authName 
-	)
+	const char * authName)
 	:
 	RadAuthBase(authName, RadAliasAuthConfigSectionName)
 {
 	m_fixedUsername = GetConfig()->GetString(
-		RadAliasAuthConfigSectionName, "FixedUsername", ""
-		);
+		RadAliasAuthConfigSectionName, "FixedUsername", "");
 	m_fixedPassword = Toolkit::Instance()->ReadPassword(
-		RadAliasAuthConfigSectionName, "FixedPassword"
-		);
+		RadAliasAuthConfigSectionName, "FixedPassword");
+	m_emptyUsername = GetConfig()->GetString(
+		RadAliasAuthConfigSectionName, "EmptyUsername", "");
 }
 
 RadAliasAuth::~RadAliasAuth()
@@ -1328,25 +1256,24 @@ RadAliasAuth::~RadAliasAuth()
 }
 
 int RadAliasAuth::AppendUsernameAndPassword(
-	RadiusPDU& pdu,
-	RasPDU<H225_RegistrationRequest>& rrqPdu, 
-	RRQAuthData& authData,
-	PString* username
+	RadiusPDU & pdu,
+	RasPDU<H225_RegistrationRequest> & rrqPdu, 
+	RRQAuthData & authData,
+	PString * username
 	) const
 {
 	const PString id = GetUsername(rrqPdu);
 	if (id.IsEmpty() && m_fixedUsername.IsEmpty()) {
 		PTRACE(3, "RADAUTH\t" << GetName() << " RRQ check failed: "
-			"neither FixedUsername nor alias inside RRQ were found"
-			);
+			"neither FixedUsername nor alias inside RRQ were found");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " RRQ check failed");
 		authData.m_rejectReason = H225_RegistrationRejectReason::e_securityDenial;
 		return GetDefaultStatus();
 	}
 	
 	// append User-Name
    	pdu.AppendAttr(RadiusAttr::UserName, 
-		m_fixedUsername.IsEmpty() ? id : m_fixedUsername
-		);
+		m_fixedUsername.IsEmpty() ? id : m_fixedUsername);
 	
 	if (username != NULL)
 		*username = (const char*)id;
@@ -1356,32 +1283,30 @@ int RadAliasAuth::AppendUsernameAndPassword(
 		pdu.AppendAttr(RadiusAttr::UserPassword, m_fixedPassword);
 	else 
 		pdu.AppendAttr(RadiusAttr::UserPassword, 
-			m_fixedUsername.IsEmpty() ? id : m_fixedUsername
-			);
+			m_fixedUsername.IsEmpty() ? id : m_fixedUsername);
 		
 	return e_ok;			
 }
 
 int RadAliasAuth::AppendUsernameAndPassword(
-	RadiusPDU& pdu,
-	RasPDU<H225_AdmissionRequest>& arqPdu,
-	ARQAuthData& authData,
-	PString* username
+	RadiusPDU & pdu,
+	RasPDU<H225_AdmissionRequest> & arqPdu,
+	ARQAuthData & authData,
+	PString * username
 	) const
 {
 	const PString id = GetUsername(arqPdu, authData);
 	if (id.IsEmpty() && m_fixedUsername.IsEmpty()) {
 		PTRACE(3, "RADAUTH\t" << GetName() << " ARQ check failed: "
-			"neither FixedUsername nor alias inside ARQ were found"
-			);
+			"neither FixedUsername nor alias inside ARQ were found");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " ARQ check failed");
 		authData.m_rejectReason = H225_AdmissionRejectReason::e_securityDenial;
 		return GetDefaultStatus();
 	}
 	
 	// append User-Name
    	pdu.AppendAttr(RadiusAttr::UserName, 
-		m_fixedUsername.IsEmpty() ? id : m_fixedUsername
-		);
+		m_fixedUsername.IsEmpty() ? id : m_fixedUsername);
 
 	if (username != NULL)
 		*username = (const char*)id;
@@ -1390,33 +1315,36 @@ int RadAliasAuth::AppendUsernameAndPassword(
 		pdu.AppendAttr(RadiusAttr::UserPassword, m_fixedPassword);
 	else
 		pdu.AppendAttr(RadiusAttr::UserPassword, 
-			m_fixedUsername.IsEmpty() ? id : m_fixedUsername
-			);
+			m_fixedUsername.IsEmpty() ? id : m_fixedUsername);
 			
 	return e_ok;
 }
 
 int RadAliasAuth::AppendUsernameAndPassword(
-	RadiusPDU& pdu,
-	SetupMsg &setup,
-	endptr& /*callingEP*/,
-	SetupAuthData& authData,
-	PString* username
+	RadiusPDU & pdu,
+	SetupMsg & setup,
+	endptr& /* callingEP*/,
+	SetupAuthData & authData,
+	PString * username
 	) const
 {
 	const PString id = GetUsername(setup, authData);
-	if (id.IsEmpty() && m_fixedUsername.IsEmpty()) {
+	if (id.IsEmpty() && m_fixedUsername.IsEmpty() && m_emptyUsername.IsEmpty()) {
 		PTRACE(3, "RADAUTH\t" << GetName() << " Setup check failed: "
-			"neither FixedUsername nor alias inside Setup were found"
-			);
+			"neither EmptyUsername nor FixedUsername nor alias inside Setup were found");
+		SNMP_TRAP(8, SNMPError, Authentication, GetName() + " Setup check failed");
 		authData.m_rejectReason = H225_ReleaseCompleteReason::e_badFormatAddress;
 		return GetDefaultStatus();
 	}
-	
+
+	// append EmptyUsername
+	if (id.IsEmpty() && !m_emptyUsername.IsEmpty()) {
+		pdu.AppendAttr(RadiusAttr::UserName, m_emptyUsername);
+	}
+
 	// append User-Name
    	pdu.AppendAttr(RadiusAttr::UserName, 
-		m_fixedUsername.IsEmpty() ? id : m_fixedUsername
-		);
+		m_fixedUsername.IsEmpty() ? id : m_fixedUsername);
 
 	if (username != NULL)
 		*username = (const char*)id;
@@ -1425,8 +1353,7 @@ int RadAliasAuth::AppendUsernameAndPassword(
 		pdu.AppendAttr(RadiusAttr::UserPassword, m_fixedPassword);
 	else
 		pdu.AppendAttr(RadiusAttr::UserPassword, 
-			m_fixedUsername.IsEmpty() ? id : m_fixedUsername
-			);
+			m_fixedUsername.IsEmpty() ? id : m_fixedUsername);
 			
 	return e_ok;
 }
