@@ -4,7 +4,7 @@
  * RADIUS protocol classes.
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
- * Copyright (c) 2003-2011, Jan Willamowius
+ * Copyright (c) 2003-2012, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -22,6 +22,7 @@
 #include <ptclib/random.h>
 #include "Toolkit.h"
 #include "h323util.h"
+#include "snmp.h"
 #include "radproto.h"
 
 // ignore overflow warnings
@@ -1363,7 +1364,8 @@ RadiusSocket::RadiusSocket(
 	}
 	m_addr = GNUGK_INADDR_ANY;
 	m_port = _port;
-	Toolkit::Instance()->PortNotification(RadiusPort, PortOpen, "udp", GNUGK_INADDR_ANY, _port);
+	if (Toolkit::Instance()->IsPortNotificationActive())
+		Toolkit::Instance()->PortNotification(RadiusPort, PortOpen, "udp", GNUGK_INADDR_ANY, _port);
 
 	PRandom random;
 	const unsigned _id_ = random;
@@ -1401,7 +1403,8 @@ RadiusSocket::RadiusSocket(
 	}
 	m_addr = addr;
 	m_port = _port;
-	Toolkit::Instance()->PortNotification(RadiusPort, PortOpen, "udp", addr, _port);
+	if (Toolkit::Instance()->IsPortNotificationActive())
+		Toolkit::Instance()->PortNotification(RadiusPort, PortOpen, "udp", addr, _port);
 
 	PRandom random;
 	const unsigned _id_ = random;
@@ -1425,7 +1428,8 @@ RadiusSocket::RadiusSocket(
 
 RadiusSocket::~RadiusSocket()
 {
-	Toolkit::Instance()->PortNotification(RadiusPort, PortClose, "udp", m_addr, m_port);
+	if (Toolkit::Instance()->IsPortNotificationActive())
+		Toolkit::Instance()->PortNotification(RadiusPort, PortClose, "udp", m_addr, m_port);
 
 	PWaitAndSignal lock(m_readMutex);
 
@@ -1508,11 +1512,13 @@ bool RadiusSocket::MakeRequest(
 			const PINDEX index = AllocReadSyncPoint();
 			if (index == P_MAX_INDEX) {
 				PTRACE(1, "RADIUS\tFailed to allocate a new mutex for the request (id:" << (PINDEX)id << ')');
+				SNMP_TRAP(8, SNMPError, Network, "Radius failed");
 				return false;
 			}
 			syncPoint = m_readSyncPoints[index];
 			if (syncPoint == NULL) {
 				PTRACE(1, "RADIUS\tFailed to allocate a new mutex for the request (id:" << (PINDEX)id << ')');
+				SNMP_TRAP(8, SNMPError, Network, "Radius failed");
 				FreeReadSyncPoint(index);
 				return false;
 			}
@@ -1531,6 +1537,7 @@ bool RadiusSocket::MakeRequest(
 			<< GetErrorCode(LastWriteError) << '/'
 			<< GetErrorNumber(LastWriteError) << ": "
 			<< GetErrorText(LastWriteError) << " (id:" << (PINDEX)id << ')');
+			SNMP_TRAP(10, SNMPError, Network, "Sending Radius message failed: " + GetErrorText(LastWriteError));
 	}
 	m_writeMutex.Signal();
 
@@ -1552,6 +1559,7 @@ bool RadiusSocket::MakeRequest(
 						<< " (" << GetErrorCode(LastReadError) << '/'
 						<< GetErrorNumber(LastReadError) << ": "
 						<< GetErrorText(LastReadError) << ')');
+				SNMP_TRAP(10, SNMPError, Network, "Radius read error: " + GetErrorText(LastReadError));
 				delete response;
 				response = NULL;
 				break;
@@ -1714,8 +1722,8 @@ bool RadiusSocket::SendRequest(
 		<< GetErrorCode(LastWriteError) << '/'
 		<< GetErrorNumber(LastWriteError) << ": "
 		<< GetErrorText(LastWriteError) << " (id:" 
-		<< (PINDEX)request->GetId() << ')'
-		);
+		<< (PINDEX)request->GetId() << ')');
+		SNMP_TRAP(10, SNMPError, Network, "Sending Radius message failed: " + GetErrorText(LastWriteError));
 	return false;
 }
 
@@ -1984,6 +1992,7 @@ bool RadiusClient::MakeRequest(
 			if (secretChanged || requireNewId || !retransmission)
 				if (!GetSocket(socket, id)) {
 					PTRACE(3, "RADIUS\tSocket allocation failed");
+					SNMP_TRAP(8, SNMPError, Network, "Radius failed");
 					delete clonedRequestPDU;
 					return false;
 				}
@@ -2021,8 +2030,8 @@ bool RadiusClient::MakeRequest(
 			if (!socket->MakeRequest(clonedRequestPDU, serverAddress, 
 					serverPort, response)) {
 				PTRACE(3, "RADIUS\tReceive response from RADIUS server failed "
-					"(id:" << (PINDEX)(clonedRequestPDU->GetId()) << ')'
-					);
+					"(id:" << (PINDEX)(clonedRequestPDU->GetId()) << ')');
+				SNMP_TRAP(8, SNMPError, Network, "Radius server failed");
 				delete clonedRequestPDU;
 				continue;
 			}
@@ -2096,6 +2105,7 @@ bool RadiusClient::SendRequest(
 
 	if (!GetSocket(socket, id)) {
 		PTRACE(3, "RADIUS\tSocket allocation failed");
+		SNMP_TRAP(8, SNMPError, Network, "Radius failed");
 		delete clonedRequestPDU;
 		return false;
 	}
@@ -2127,6 +2137,7 @@ bool RadiusClient::SendRequest(
 
 	if (!socket->SendRequest(clonedRequestPDU, serverAddress, serverPort)) {
 		PTRACE(3, "RADIUS\tError sending RADIUS request (id:" << (PINDEX)id << ')');
+		SNMP_TRAP(10, SNMPError, Network, "Sending Radius message failed");
 		delete clonedRequestPDU;
 		return false;
 	}
@@ -2246,7 +2257,7 @@ bool RadiusClient::GetSocket(RadiusSocket*& socket, unsigned char& id)
 			else
 				newSocket = CreateSocket(m_localAddress, p);
 				
-			if (newSocket->IsOpen())
+			if (newSocket && newSocket->IsOpen())
 				break;
 		}
 			
