@@ -262,7 +262,7 @@ void NetSNMPAgent::Stop()
 
 void NetSNMPAgent::Run()
 {
-	bool standalone = Toolkit::AsBool(GkConfig()->GetString(SNMPSection, "Standalone", "0"));
+	bool standalone = GkConfig()->GetBoolean(SNMPSection, "Standalone", false);
 
 	// enable Net-SNMP logging via PTRACE
 	m_logger = netsnmp_register_loghandler(NETSNMP_LOGHANDLER_CALLBACK, LOG_DEBUG);
@@ -425,12 +425,13 @@ PBoolean PTLibSNMPAgent::OnSetRequest(PINDEX reqID, PSNMP::BindingList & vars, P
 	return PFalse;	// doesn't work
 
 /*
-	// TODO: PSNMPServer::ProcessPDU() must send a response and the decoding of SET values is broken
+	// TODO: SET operation is broken in PTLib PSNMPServer
+	// PSNMPServer::ProcessPDU() must send a response and the decoding of SET values is broken
 	for(PSNMP::BindingList::iterator i = vars.begin(); i != vars.end(); ++i){
 		if (i->first == TraceLevelOIDStr + PString(".0")) {
 			if (i->second.GetObject().GetTag() == PASN_Object::UniversalInteger) {
 				PASN_Integer & num = (PASN_Integer &)(i->second.GetObject());
-				PTRACE(0, "JW SET " << i->first << " to " << num << " = " << num.GetValue());	// TODO / BUG
+				PTRACE(0, "JW SET " << i->first << " to " << num << " = " << num.GetValue());
 				//PTrace::SetLevel(num.GetValue());
 				return PTrue;
 			} else {
@@ -440,7 +441,7 @@ PBoolean PTLibSNMPAgent::OnSetRequest(PINDEX reqID, PSNMP::BindingList & vars, P
 		} else if (i->first == CatchAllOIDStr + PString(".0")) {
 			if (i->second.GetObject().GetTag() == PASN_Object::UniversalOctetString) {
 				PASN_OctetString & str = (PASN_OctetString &)(i->second.GetObject());
-				PTRACE(0, "JW SET " << i->first << " to " << str << " = " << str.GetValue());	// TODO / BUG
+				PTRACE(0, "JW SET " << i->first << " to " << str << " = " << str.GetValue());
 				return PTrue;
 			} else {
 				PTRACE(1, "SNMP\tWrong data type for SET " << i->first);
@@ -667,7 +668,11 @@ void WindowsSNMPAgent::SendWindowsSNMPTrap(unsigned trapNumber, SNMPLevel severi
 
 PCaselessString SelectSNMPImplementation() 
 {
+#ifdef HAS_NETSNMP
 	PCaselessString implementation = GkConfig()->GetString(SNMPSection, "Implementation", "Net-SNMP");
+#else
+	PCaselessString implementation = GkConfig()->GetString(SNMPSection, "Implementation", "PTLib");
+#endif
 
 	// switch to other implementation if only one is available
 #ifndef HAS_NETSNMP
@@ -694,7 +699,7 @@ PCaselessString SelectSNMPImplementation()
 void SendSNMPTrap(unsigned trapNumber, SNMPLevel severity, SNMPGroup group, const PString & msg)
 {
 	if ((severity == SNMPWarning)
-		&& !Toolkit::AsBool(GkConfig()->GetString(SNMPSection, "EnableWarningTraps", "0"))) {
+		&& !GkConfig()->GetBoolean(SNMPSection, "EnableWarningTraps", false)) {
 		// don't throw warning trap
 		return;
 	}
@@ -742,27 +747,17 @@ void StartSNMPAgent()
 
 void StopSNMPAgent() 
 {
-	PCaselessString implementation = SelectSNMPImplementation();
 #ifdef HAS_NETSNMP
-	if (implementation == "Net-SNMP") {
-		if (NetSNMPAgent::InstanceExists()) {
-			delete NetSNMPAgent::Instance();
-		}
-		return;
+	if (NetSNMPAgent::InstanceExists()) {
+		NetSNMPAgent::Instance()->Stop();
 	}
 #endif
 #ifdef HAS_PTLIBSNMP
-	if (implementation == "PTLib") {
-		// nothing to do
-		return;
-	}
+	// nothing to do
 #endif
 #ifdef HAS_WINSNMP
-	if (implementation == "Windows") {
-		if (WindowsSNMPAgent::InstanceExists()) {
-			WindowsSNMPAgent::Instance()->Stop();
-		}
-		return;
+	if (WindowsSNMPAgent::InstanceExists()) {
+		WindowsSNMPAgent::Instance()->Stop();
 	}
 #endif
 }
