@@ -14,7 +14,7 @@
  */
 
 #ifndef SQLACCT_H
-#define SQLACCT_H "@(#) $Id: sqlacct.h,v 1.11 2010/10/25 15:01:51 willamowius Exp $"
+#define SQLACCT_H "@(#) $Id: sqlacct.h,v 1.12 2012/02/12 17:21:27 shorne Exp $"
 
 #include "gkacct.h"
 
@@ -91,6 +91,10 @@ private:
 	PString m_registerQuery;
 	/// parametrized query string for endpoint un-registration
 	PString m_unregisterQuery;
+	/// parametrized query string for gatekeeper coming online
+	PString m_onQuery;
+	/// parametrized query string for gatekeeper going offline
+	PString m_offQuery;
 	/// timestamp formatting string
 	PString m_timestampFormat;
 };
