@@ -3,7 +3,7 @@
 // yasocket.cxx
 //
 // Copyright (c) Citron Network Inc. 2002-2003
-// Copyright (c) 2004-2012, Jan Willamowius
+// Copyright (c) 2004-2013, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -42,6 +42,8 @@ const long SOCKET_CHUNK_PAUSE = 250;	// send in 10K chunks
 const int MAX_SOCKET_CHUNK = 10240;	// send in 10K chunks
 }
 
+int g_maxSocketQueue = 100;	// set with [Gatekeeper::Main] MaxSocketQueue=
+
 #ifdef LARGE_FDSET
 
 bool YaSelectList::Select(SelectType t, const PTimeInterval & timeout)
@@ -128,9 +130,13 @@ bool YaSelectList::Select(SelectType t, const PTimeInterval & timeout)
 
 
 // class YaSocket
-YaSocket::YaSocket() : os_handle(-1)
+YaSocket::YaSocket() : os_handle(-1), port(0)
 {
 	lastReadCount = lastWriteCount = 0;
+	for (PINDEX i = 0; i < PSocket::NumErrorGroups; i++) {
+		lastErrorCode[i] = PSocket::NoError;
+		lastErrorNumber[i] = 0;
+	}
 }
 
 YaSocket::~YaSocket()
@@ -260,8 +266,7 @@ bool YaSocket::GetOption(int option, int & value, int level)
 bool YaSocket::GetOption(int option, void * valuePtr, PINDEX valueSize, int level)
 {
 	return ConvertOSError(::getsockopt(os_handle, level, option,
-		(char *)valuePtr, (socklen_t *)&valueSize)
-		);
+		(char *)valuePtr, (socklen_t *)&valueSize));
 }
 
 PString YaSocket::GetErrorText(PSocket::ErrorGroup group) const
@@ -804,7 +809,6 @@ USocket::USocket(IPSocket * s, const char * t)
 
 USocket::~USocket()
 {
-	//PWaitAndSignal lock(writeMutex);
 	{
 		PWaitAndSignal lock(queueMutex);
 		DeleteObjectsInContainer(queue);
@@ -864,8 +868,7 @@ bool USocket::WriteData(const BYTE * buf, int len)
 		) {
 		PWaitAndSignal lock(writeMutex);
 		while (remaining > 0) {
-			int sendnow = remaining > MAX_SOCKET_CHUNK
-				? MAX_SOCKET_CHUNK : remaining;
+			int sendnow = remaining > MAX_SOCKET_CHUNK ? MAX_SOCKET_CHUNK : remaining;
 			if (!InternalWriteData(buf, sendnow)) {
 				unsigned bytesSent = self->GetLastWriteCount();
 				remaining -= bytesSent;
@@ -881,7 +884,7 @@ bool USocket::WriteData(const BYTE * buf, int len)
 		if (remaining == 0)
 			return true;
 	}
-	if (qsize > 100) { // to be justitied
+	if (qsize > g_maxSocketQueue) {
 		PTRACE(2, type << '\t' << Name() << " is dead and closed");
 		CloseSocket();
 	} else if (remaining > 0 && IsSocketOpen()) {
@@ -970,8 +973,6 @@ SocketsReader::~SocketsReader()
 {
 	RemoveClosed(false);
 	SocketsReader::CleanUp();
-	//DeleteObjectsInContainer(m_removed);
-	//DeleteObjectsInContainer(m_sockets);
 }
 
 void SocketsReader::Stop()
@@ -1068,7 +1069,6 @@ void SocketsReader::Exec()
 	} else {
 		CleanUp();
 		ConfigReloadMutex.EndRead();
-		//PTRACE(6, GetName() << " waiting...");
 		Wait(SOCKETSREADER_IDLE_TIMEOUT);
 		ConfigReloadMutex.StartRead();
 	}
@@ -1173,6 +1173,9 @@ void TCPServer::ReadSocket(IPSocket * socket)
 	}
 
 	TCPListenSocket *listener = dynamic_cast<TCPListenSocket *>(socket);
+	if (!listener)
+		return;
+
 	ServerSocket *acceptor = listener->CreateAcceptor();
 	if (acceptor->Accept(*listener)) {
 		PTRACE(6, GetName() << "\tAccepted new connection on " << socket->GetName() << " from " << acceptor->GetName());
